// SKY
#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 0

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct SH9
{
    vec3 L[9];
};

struct FlrInput
{
    vec3 normal;
    vec2 uv0;
    vec3 posWS;
    vec4 pos;
    vec4 prevPos;
};

struct FlrOutput
{
    vec4 albedo;
    vec2 motionVector;
};

struct SH9Compressed
{
    uvec4 data0[2];
    uvec4 data1;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 25, std140) uniform PerView
{
    vec4 skyModelConfig[7];
    vec4 skyModelRadiances;
    vec4 cameraPosWS;
    vec4 dirLightDirWS;
    vec4 dirLightColorAndIntens;
    mat4 invViewProjMatrix;
    SH9Compressed skyModelSH;
    vec4 dynamicResolutionParameters;
    vec4 projectionOffsets;
} _542;

layout(set = 0, binding = 26, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 paramData0;
    vec4 paramData1;
    vec4 paramData2;
    vec4 paramData3;
    vec4 paramData4;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _791;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _1023;

layout(set = 0, binding = 20) uniform SAMPLER_2D backgroundLayer0Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D cloudLayer0Tex;
layout(set = 0, binding = 22) uniform SAMPLER_2D volumetricCloudsTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D waterTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 4) in vec3 inPosWS;
layout(location = 6) in vec4 inPosition;
layout(location = 7) in vec4 inPrevPosition;
layout(location = 0) out vec4 outAlbedo;
layout(location = 1) out vec4 outMotionVector;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;

float getSkyModelConfig(uint channelIdx, uint configIdx)
{
    uint idx = configIdx + (channelIdx * 9u);
    uvec2 packedDataOffset = uvec2(idx / 4u, idx % 4u);
    return _542.skyModelConfig[packedDataOffset.x][packedDataOffset.y];
}

vec3 calculateSkyModelRadiance(vec3 theta, vec3 gamma)
{
    vec3 configuration[9];
    for (uint i = 0u; i < 9u; i++)
    {
        uint param = 0u;
        uint param_1 = i;
        uint param_2 = 1u;
        uint param_3 = i;
        uint param_4 = 2u;
        uint param_5 = i;
        configuration[i] = vec3(getSkyModelConfig(param, param_1), getSkyModelConfig(param_2, param_3), getSkyModelConfig(param_4, param_5));
    }
    vec3 expM = exp(configuration[4] * gamma);
    vec3 rayM = cos(gamma) * cos(gamma);
    vec3 mieM = (vec3(1.0) + (cos(gamma) * cos(gamma))) / pow(abs((vec3(1.0) + (configuration[8] * configuration[8])) - ((configuration[8] * 2.0) * cos(gamma))), vec3(1.5));
    vec3 zenith = sqrt(cos(theta));
    return max((vec3(1.0) + (configuration[0] * exp(configuration[1] / (cos(theta) + vec3(0.00999999977648258209228515625))))) * ((((configuration[2] + (configuration[3] * expM)) + (configuration[5] * rayM)) + (configuration[6] * mieM)) + (configuration[7] * zenith)), vec3(0.0));
}

float gammaToLinear(float color)
{
    return pow(clamp(color, 0.0, 1.0), 2.2000000476837158203125);
}

vec4 unpackUNorm4x8(uint p_PackedValue)
{
    return vec4(float(p_PackedValue & 255u), float((p_PackedValue >> uint(8)) & 255u), float((p_PackedValue >> uint(16)) & 255u), float((p_PackedValue >> uint(24)) & 255u)) * 0.00392160005867481231689453125;
}

SH9 decodeSH9(vec4 data[7])
{
    SH9 result;
    result.L[0] = data[0].xyz;
    result.L[1] = vec3(data[0].w, data[1].xy);
    result.L[2] = vec3(data[1].zw, data[2].x);
    result.L[3] = data[2].yzw;
    result.L[4] = data[3].xyz;
    result.L[5] = vec3(data[3].w, data[4].xy);
    result.L[6] = vec3(data[4].zw, data[5].x);
    result.L[7] = data[5].yzw;
    result.L[8] = data[6].xyz;
    return result;
}

SH9 decodeSH9(uvec4 data0[2], uvec4 data1)
{
    uint param = data0[0].x;
    vec4 tempCoeff = (unpackUNorm4x8(param) * 2.0) - vec4(1.0);
    float tempCoeffLen = unpackHalf2x16(data1.x & 65535u).x;
    vec4 coeffs[7];
    coeffs[0] = tempCoeff * tempCoeffLen;
    uint param_1 = data0[0].y;
    tempCoeff = (unpackUNorm4x8(param_1) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16((data1.x >> uint(16)) & 65535u).x;
    coeffs[1] = tempCoeff * tempCoeffLen;
    uint param_2 = data0[0].z;
    tempCoeff = (unpackUNorm4x8(param_2) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16(data1.y & 65535u).x;
    coeffs[2] = tempCoeff * tempCoeffLen;
    uint param_3 = data0[0].w;
    tempCoeff = (unpackUNorm4x8(param_3) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16((data1.y >> uint(16)) & 65535u).x;
    coeffs[3] = tempCoeff * tempCoeffLen;
    uint param_4 = data0[1].x;
    tempCoeff = (unpackUNorm4x8(param_4) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16(data1.z & 65535u).x;
    coeffs[4] = tempCoeff * tempCoeffLen;
    uint param_5 = data0[1].y;
    tempCoeff = (unpackUNorm4x8(param_5) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16((data1.z >> uint(16)) & 65535u).x;
    coeffs[5] = tempCoeff * tempCoeffLen;
    uint param_6 = data0[1].z;
    tempCoeff = (unpackUNorm4x8(param_6) * 2.0) - vec4(1.0);
    tempCoeffLen = unpackHalf2x16(data1.w & 65535u).x;
    coeffs[6] = tempCoeff * tempCoeffLen;
    vec4 param_7[7] = coeffs;
    return decodeSH9(param_7);
}

SH9 projectOntoSH9(vec3 n)
{
    SH9 sh;
    sh.L[0] = vec3(0.88622760772705078125);
    vec3 a1 = n.yzx * 0.48860299587249755859375;
    sh.L[1] = vec3(2.094399929046630859375) * a1.x;
    sh.L[2] = vec3(2.094399929046630859375) * a1.y;
    sh.L[3] = vec3(2.094399929046630859375) * a1.z;
    sh.L[4] = vec3(0.785398006439208984375) * ((1.09254801273345947265625 * n.x) * n.y);
    sh.L[5] = vec3(0.785398006439208984375) * ((1.09254801273345947265625 * n.y) * n.z);
    sh.L[6] = vec3(0.785398006439208984375) * (0.3153919875621795654296875 * (((3.0 * n.z) * n.z) - 1.0));
    sh.L[7] = vec3(0.785398006439208984375) * ((1.09254801273345947265625 * n.x) * n.z);
    sh.L[8] = vec3(0.785398006439208984375) * (0.546274006366729736328125 * ((n.x * n.x) - (n.y * n.y)));
    return sh;
}

vec3 shDotProduct(SH9 a, SH9 b)
{
    vec3 result = a.L[0] * b.L[0];
    result += (a.L[1] * b.L[1]);
    result += (a.L[2] * b.L[2]);
    result += (a.L[3] * b.L[3]);
    result += (a.L[4] * b.L[4]);
    result += (a.L[5] * b.L[5]);
    result += (a.L[6] * b.L[6]);
    result += (a.L[7] * b.L[7]);
    result += (a.L[8] * b.L[8]);
    return result;
}

vec3 sampleSH(uvec4 data0[2], uvec4 data1, vec3 dir)
{
    uvec4 param[2] = data0;
    uvec4 param_1 = data1;
    SH9 sh = decodeSH9(param, param_1);
    vec3 param_2 = dir;
    SH9 dirSH = projectOntoSH9(param_2);
    SH9 param_3 = dirSH;
    SH9 param_4 = sh;
    return max(shDotProduct(param_3, param_4), vec3(0.0));
}

vec2 getMotionVector(vec4 currPos, vec4 prevPos, vec4 projectionOffsets)
{
    vec2 currPosNDC = (currPos.xy / vec2(currPos.w)) + projectionOffsets.xy;
    vec2 prevPosNDC = (prevPos.xy / vec2(prevPos.w)) + projectionOffsets.zw;
    currPosNDC.y = 1.0 - currPosNDC.y;
    prevPosNDC.y = 1.0 - prevPosNDC.y;
    vec2 motionVector = (prevPosNDC - currPosNDC) * vec2(0.5, -0.5);
    motionVector = clamp(motionVector, vec2(-1.0), vec2(1.0));
    return motionVector;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec3 V = normalize(flrInput.posWS - _542.cameraPosWS.xyz);
    float theta = acos(max(dot(V, vec3(0.0, 1.0, 0.0)), 9.9999999392252902907785028219223e-09));
    float gamma = acos(max(dot(V, _542.dirLightDirWS.xyz), 9.9999997473787516355514526367188e-05));
    vec3 param = vec3(theta);
    vec3 param_1 = vec3(gamma);
    vec3 computedSkyModelRadiance = calculateSkyModelRadiance(param, param_1) * _542.skyModelRadiances.xyz;
    flrOutput.albedo = vec4(0.0);
    vec3 _706 = flrOutput.albedo.xyz + computedSkyModelRadiance;
    flrOutput.albedo = vec4(_706.x, _706.y, _706.z, flrOutput.albedo.w);
    vec3 _732 = flrOutput.albedo.xyz + ((_542.dirLightColorAndIntens.xyz * _542.dirLightColorAndIntens.w) * float((max(dot(_542.dirLightDirWS.xyz, V) - 0.999000012874603271484375, 0.0) * 5000.0) > 0.5));
    flrOutput.albedo = vec4(_732.x, _732.y, _732.z, flrOutput.albedo.w);
    vec4 bgLayer0Color = SAMPLE_TEXTURE(backgroundLayer0Tex, flrInput.uv0);
    vec4 cloudLayer0Color = SAMPLE_TEXTURE(cloudLayer0Tex, flrInput.uv0);
    float param_2 = cloudLayer0Color.w;
    cloudLayer0Color.w = gammaToLinear(param_2);
    uvec4 param_3[2];
    param_3[0] = _542.skyModelSH.data0[0];
    param_3[1] = _542.skyModelSH.data0[1];
    uvec4 param_4 = _542.skyModelSH.data1;
    vec3 param_5 = V;
    vec3 irrad = sampleSH(param_3, param_4, param_5);
    vec3 _781 = mix(bgLayer0Color.xyz, bgLayer0Color.xyz * irrad, vec3(_542.skyModelRadiances.w));
    bgLayer0Color = vec4(_781.x, _781.y, _781.z, bgLayer0Color.w);
    float param_6 = bgLayer0Color.w;
    bgLayer0Color.w = gammaToLinear(param_6);
    bgLayer0Color.w *= _791.paramData0.x;
    vec3 _806 = cloudLayer0Color.xyz * (_791.paramData1.xyz * _791.paramData0.z);
    cloudLayer0Color = vec4(_806.x, _806.y, _806.z, cloudLayer0Color.w);
    cloudLayer0Color.w *= _791.paramData0.y;
    vec3 _823 = mix(flrOutput.albedo.xyz, cloudLayer0Color.xyz, vec3(cloudLayer0Color.w));
    flrOutput.albedo = vec4(_823.x, _823.y, _823.z, flrOutput.albedo.w);
    vec2 cloudsUV = ((flrInput.pos.xy / vec2(flrInput.pos.w)) * 0.5) + vec2(0.5);

	// Fix for cloudsUV;
	cloudsUV.x -= vk3d_params[VIEW].stereo.x*0.5;

    vec4 clouds = SAMPLE_TEXTURE(volumetricCloudsTex, cloudsUV);
    vec3 _851 = (flrOutput.albedo.xyz * clouds.w) + clouds.xyz;
    flrOutput.albedo = vec4(_851.x, _851.y, _851.z, flrOutput.albedo.w);

     // Fix for water
    cloudsUV.x += vk3d_params[VIEW].stereo.x*0.5;
    
    vec4 water = SAMPLE_TEXTURE(waterTex, cloudsUV * _542.dynamicResolutionParameters.xy);
    if (water.w > 0.0)
    {
        float horizonFade = 1.0;
        if (_791.paramData8.z < _791.paramData8.y)
        {
            horizonFade = smoothstep(_791.paramData8.z, _791.paramData8.y, 1.0 - flrInput.uv0.y);
        }
        vec3 _896 = mix(flrOutput.albedo.xyz, water.xyz, vec3(water.w * horizonFade));
        flrOutput.albedo = vec4(_896.x, _896.y, _896.z, flrOutput.albedo.w);
        vec3 _908 = mix(flrOutput.albedo.xyz, bgLayer0Color.xyz, vec3(bgLayer0Color.w));
        flrOutput.albedo = vec4(_908.x, _908.y, _908.z, flrOutput.albedo.w);
        float waterFade = water.w;
        waterFade *= mix(1.0 - bgLayer0Color.w, 1.0, smoothstep(0.675000011920928955078125, 0.699999988079071044921875, flrInput.uv0.y));
        vec3 _935 = mix(flrOutput.albedo.xyz, water.xyz, vec3(waterFade * horizonFade));
        flrOutput.albedo = vec4(_935.x, _935.y, _935.z, flrOutput.albedo.w);
    }
    else
    {
        vec3 _948 = mix(flrOutput.albedo.xyz, bgLayer0Color.xyz, vec3(bgLayer0Color.w));
        flrOutput.albedo = vec4(_948.x, _948.y, _948.z, flrOutput.albedo.w);
    }
    vec4 param_7 = flrInput.pos;
    vec4 param_8 = flrInput.prevPos;
    vec4 param_9 = _542.projectionOffsets;
    flrOutput.motionVector = getMotionVector(param_7, param_8, param_9);
}

void main()
{
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal;
    flrInput.posWS = inPosWS;
    flrInput.pos = inPosition;
    flrInput.prevPos = inPrevPosition;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outAlbedo = flrOutput.albedo;
    outMotionVector = vec4(flrOutput.motionVector.x, flrOutput.motionVector.y, outMotionVector.z, outMotionVector.w);
    outMotionVector = vec4(outMotionVector.x, outMotionVector.y, vec2(0.0).x, vec2(0.0).y);
}






