//TAA. Temporal antialiasing.
#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

const ivec2 _132[4] = ivec2[](ivec2(-1), ivec2(1, -1), ivec2(-1, 1), ivec2(1));

// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection

struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};


   

layout(set = 0, binding = 2, std140) uniform PerView
{
    vec4 dynamicResolutionParameters;
    vec4 backbufferSize;
} _171;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _217;

layout(set = 0, binding = 1) uniform SAMPLER_2D depthTex;
layout(set = 0, binding = 0) uniform SAMPLER_2D motionVectorsTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 0) in vec2 inUV0;
layout(location = 0) out float outTempHistData;

vec2 getScreenNearestDepthScreenUV(SAMPLER_2D depthTex_1, vec2 screenUV, vec2 fragmentSize)
{
    vec2 screenUVmod = screenUV;
    float dc = TEXTURE_LOD(depthTex_1, screenUV, 0.0).x;
    float nearestDepth = 1.0;
    for (uint i = 0u; i < 4u; i++)
    {
        vec2 neighborScreenUV = screenUV - (vec2(_132[i]) * fragmentSize);
        float neighborDepth = TEXTURE_LOD(depthTex_1, neighborScreenUV, 0.0).x;
        float currDepthDiff = abs(dc - neighborDepth);
        if (currDepthDiff < nearestDepth)
        {
            nearestDepth = currDepthDiff;
            screenUVmod = neighborScreenUV;
        }
    }
    return screenUVmod;
}

uint decodeMaterialId(float source)
{
    return packSnorm2x16(vec2(source, 0.0));
}

uint unpackUInt16(float val)
{
    return uint(val / 1.52587890625e-05);
}

uint getTemporalHistoryData(vec4 motionVector)
{
    float param = motionVector.z;
    uint materialId = decodeMaterialId(param);
    float param_1 = motionVector.w;
    uint packedIntegerParams = unpackUInt16(param_1);
    uint meshInstId = packedIntegerParams & 16383u;
    bool dynamicObj = meshInstId > 0u;
    bool taaGhostFlag = (packedIntegerParams & 16384u) > 0u;
    uint tempHistData = 0u;
    tempHistData |= uint(int(dynamicObj));
    tempHistData |= uint(taaGhostFlag ? 2 : 0);
    return tempHistData;
}

uint getTemporalHistoryData(SAMPLER_2D motionVectorsTex_1, vec2 screenUV)
{
    vec4 motionVector = TEXTURE_LOD(motionVectorsTex_1, screenUV, 0.0);
    vec4 param = motionVector;
    return getTemporalHistoryData(param);
}

void main()
{
    vec2 screenUV = inUV0 * _171.dynamicResolutionParameters.xy;
    
    //flrInput.uv0.x -= vk3d_params[VIEW].stereo.x *(1-vk3d_params[VIEW].stereo.y*depth*0.2)*0.5;
    
    vec2 param = screenUV;
    vec2 param_1 = _171.backbufferSize.xy;
    screenUV = getScreenNearestDepthScreenUV(depthTex, param, param_1);
    vec2 param_2 = screenUV;
    uint tempHistData = getTemporalHistoryData(motionVectorsTex, param_2);
    outTempHistData = float(tempHistData) / 255.0;
}


