#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec3 posWS;
    vec3 normal;
    vec3 binormal;
    vec4 tangent;
    vec2 uv0;
    vec4 pos;
    vec2 dynamicResolutionFactor;
};

struct FlrOutput
{
    vec4 albedo;
    vec4 normals;
    vec4 parameter;
    vec3 normalsTS;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 31, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 tint;
    vec4 waterParams4;
    vec4 uvTransform;
    vec4 waterParams0;
    vec4 waterParams3;
    vec4 waterParams1;
    vec4 waterParams2;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
} _289;

layout(set = 0, binding = 30, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _344;

layout(set = 0, binding = 32, std140) uniform PerFrame
{
    vec4 frameParameters0;
    vec4 nearFar;
} _399;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _1084;

layout(set = 0, binding = 27) uniform SAMPLER_2D depthBufferTex;
layout(set = 0, binding = 22) uniform SAMPLER_2D waterFlowTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D waterNoiseTex;
layout(set = 0, binding = 24) uniform SAMPLER_2D waterNormal0Tex;
layout(set = 0, binding = 25) uniform SAMPLER_2D waterNormal1Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D waterFoamTex;
layout(set = 0, binding = 20) uniform SAMPLER_2D waterDebrisTex;
layout(set = 0, binding = 26) uniform SAMPLER_2D waterParameterTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 4) in vec3 inPosWS;
layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;
layout(location = 0) out vec4 outAlbedo;
layout(location = 1) out vec4 outNormals;
layout(location = 2) out vec4 outParameter;
layout(location = 3) out vec4 outMotionVector;

vec4 decodeARGB8uint(uint colorU)
{
    return vec4(float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375, float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375);
}

vec3 gammaToLinear(vec3 color)
{
    return pow(clamp(color, vec3(0.0), vec3(1.0)), vec3(2.2000000476837158203125));
}

vec4 decodeRGBA8uint(uint colorU)
{
    return vec4(float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375, float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375);
}

float oldDeviceDepthToLinear(float z, float near, float far)
{
    float a = far / (far - near);
    float b = ((-far) * near) / (far - near);
    return b / ((1.0 - z) - a);
}

vec3 decodeNormalMap(vec4 normalSample)
{
    vec2 _126 = (normalSample.wy - vec2(0.5)) * 2.0;
    vec3 normal;
    normal = vec3(_126.x, _126.y, normal.z);
    normal.z = sqrt(1.0 - ((normal.x * normal.x) - (normal.y * normal.y)));
    return normal;
}

vec3 getAnimatedWaterNormal(vec2 uv, float timeOffset, float normalBias, SAMPLER_2D normal0Tex, SAMPLER_2D normal1Tex)
{
    vec2 offset = vec2(timeOffset, 0.0);
    vec3 waterNormal = vec3(0.0);
    vec4 param = SAMPLE_TEXTURE1(normal0Tex, uv + offset, normalBias);
    waterNormal += decodeNormalMap(param);
    vec4 param_1 = SAMPLE_TEXTURE1(normal1Tex, uv - offset, normalBias);
    waterNormal += decodeNormalMap(param_1);
    vec4 param_2 = SAMPLE_TEXTURE1(normal0Tex, (vec2(0.5) - uv) + offset.yx, normalBias);
    waterNormal += decodeNormalMap(param_2);
    vec4 param_3 = SAMPLE_TEXTURE1(normal1Tex, (vec2(0.5) - uv) - offset.yx, normalBias);
    waterNormal += decodeNormalMap(param_3);
    return waterNormal;
}

vec2 signNotZero(vec2 v)
{
    return vec2((v.x >= 0.0) ? 1.0 : (-1.0), (v.y >= 0.0) ? 1.0 : (-1.0));
}

vec2 encodeNormal(vec3 v)
{
    vec2 p = v.xy * (1.0 / ((abs(v.x) + abs(v.y)) + abs(v.z)));
    vec2 _102;
    if (v.z <= 0.0)
    {
        vec2 param = p;
        _102 = (vec2(1.0) - abs(p.yx)) * signNotZero(param);
    }
    else
    {
        _102 = p;
    }
    return _102;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    uint param = floatBitsToUint(_289.tint.x);
    vec4 debrisTint = decodeARGB8uint(param);
    uint param_1 = floatBitsToUint(_289.tint.y);
    vec4 foamTint = decodeARGB8uint(param_1);
    uint param_2 = floatBitsToUint(_289.tint.z);
    vec4 fogColor = decodeARGB8uint(param_2);
    vec3 param_3 = debrisTint.xyz;
    vec3 _312 = gammaToLinear(param_3);
    debrisTint = vec4(_312.x, _312.y, _312.z, debrisTint.w);
    vec3 param_4 = foamTint.xyz;
    vec3 _318 = gammaToLinear(param_4);
    foamTint = vec4(_318.x, _318.y, _318.z, foamTint.w);
    vec3 param_5 = fogColor.xyz;
    vec3 _324 = gammaToLinear(param_5);
    fogColor = vec4(_324.x, _324.y, _324.z, fogColor.w);
    float normalStrength = _289.waterParams3.x;
    float translucency = fogColor.w;
    flrOutput.albedo = vec4(fogColor.xyz.x, fogColor.xyz.y, fogColor.xyz.z, flrOutput.albedo.w);
    uint param_6 = floatBitsToUint(_344.materialParams0.w);
    vec3 perInstanceEmissiveTint = decodeRGBA8uint(param_6).xyz;
    vec3 waterEmissiveTint = perInstanceEmissiveTint * _344.materialParams0.y;
    vec3 _360 = flrOutput.albedo.xyz * waterEmissiveTint;
    flrOutput.albedo = vec4(_360.x, _360.y, _360.z, flrOutput.albedo.w);
    vec2 uv = (flrInput.uv0 * _289.uvTransform.xy) + _289.uvTransform.zw;
    vec2 uvFoam = uv;
    if ((_289.paramIntData0.x & 8u) > 0u)
    {
        uv = (flrInput.uv0 * _289.uvTransform.xy) + (_289.uvTransform.zw * _399.frameParameters0.x);
        uvFoam = (flrInput.uv0 * _289.uvTransform.xy) + ((_289.uvTransform.zw * _399.frameParameters0.x) * _289.waterParams3.w);
    }
    vec2 waterDUDV = vec2(sin((_399.frameParameters0.x * 1.25) + uv.x), cos((_399.frameParameters0.x * 0.75) + uv.y)) * _289.waterParams2.y;
    float waterDepth = 0.0;
    float softShoreAmount = 1.0;
    if ((_289.paramIntData0.x & 1u) > 0u)
    {
        vec2 posNDC = flrInput.pos.xy / vec2(flrInput.pos.w);
        vec2 screenUV = (posNDC * 0.5) + vec2(0.5);
        screenUV *= flrInput.dynamicResolutionFactor;
        float waterSurfaceDepth = flrInput.pos.z / flrInput.pos.w;
        float param_7 = waterSurfaceDepth;
        float param_8 = _399.nearFar.x;
        float param_9 = _399.nearFar.y;
        waterSurfaceDepth = oldDeviceDepthToLinear(param_7, param_8, param_9);
        waterSurfaceDepth /= _399.nearFar.y;
        float solidSurfaceDepth = SAMPLE_TEXTURE(depthBufferTex, screenUV).x;
        float param_10 = solidSurfaceDepth;
        float param_11 = _399.nearFar.x;
        float param_12 = _399.nearFar.y;
        solidSurfaceDepth = oldDeviceDepthToLinear(param_10, param_11, param_12);
        solidSurfaceDepth /= _399.nearFar.y;
        waterDepth = solidSurfaceDepth - waterSurfaceDepth;
        waterDepth *= _399.nearFar.y;
        softShoreAmount = clamp(waterDepth / max(_289.waterParams1.y, 9.9999999392252902907785028219223e-09), 0.0, 1.0);
        normalStrength *= softShoreAmount;
        waterDUDV *= clamp(waterDepth / max(_289.waterParams1.w, 9.9999999392252902907785028219223e-09), 0.0, 1.0);
        translucency *= (1.0 - exp(((-waterDepth) * _289.waterParams1.z) * 0.00999999977648258209228515625));
    }
    vec2 baseUV = uv + waterDUDV;
    float flowIntensity = 0.0;
    float flowPhaseBlendFactor = 0.0;
    vec2 flowUV0 = vec2(0.0);
    vec2 flowUV1 = vec2(0.0);
    vec2 flowUV0Normal = vec2(0.0);
    vec2 flowUV1Normal = vec2(0.0);
    if ((_289.paramIntData0.x & 2u) > 0u)
    {
        vec2 waterFlowUV = uv * _289.waterParams1.x;
        vec2 waterFlowVector = SAMPLE_TEXTURE(waterFlowTex, waterFlowUV).wy;
        waterFlowVector = (waterFlowVector * 2.0) - vec2(1.0);
        flowIntensity = length(waterFlowVector);
        normalStrength *= (1.0 - min(flowIntensity, _289.waterParams0.w));
        waterDUDV *= (1.0 - clamp(flowIntensity, 0.0, 1.0));
        baseUV = uv + waterDUDV;
        float waterPhaseTimer = _289.waterParams0.y * (_399.frameParameters0.x + SAMPLE_TEXTURE(waterNoiseTex, waterFlowUV).y);
        float waterPhase1 = mod(waterPhaseTimer, 2.0);
        flowPhaseBlendFactor = abs(waterPhase1 - 1.0);
        waterFlowVector *= _289.waterParams0.z;
        flowUV0Normal = waterFlowVector * waterPhase1;
        float waterPhase2 = mod(waterPhaseTimer + 1.0, 2.0);
        flowUV1Normal = (waterFlowVector * waterPhase2) + vec2(0.5);
        waterPhase1 -= 1.0;
        waterPhase2 -= 1.0;
        flowUV0 = waterFlowVector * waterPhase1;
        flowUV1 = (waterFlowVector * waterPhase2) + vec2(0.5);
    }
    float animationTime = _399.frameParameters0.x * _289.waterParams0.x;
    vec3 normal;
    if ((_289.paramIntData0.x & 2u) > 0u)
    {
        vec2 param_13 = baseUV + flowUV0;
        float param_14 = animationTime;
        float param_15 = _289.waterParams3.y;
        vec2 param_16 = baseUV + flowUV1;
        float param_17 = animationTime;
        float param_18 = _289.waterParams3.y;
        normal = mix(getAnimatedWaterNormal(param_13, param_14, param_15, waterNormal0Tex, waterNormal1Tex), getAnimatedWaterNormal(param_16, param_17, param_18, waterNormal0Tex, waterNormal1Tex), vec3(flowPhaseBlendFactor));
    }
    else
    {
        vec2 param_19 = baseUV;
        float param_20 = animationTime;
        float param_21 = _289.waterParams3.y;
        normal = getAnimatedWaterNormal(param_19, param_20, param_21, waterNormal0Tex, waterNormal1Tex);
    }
    normal = mix(vec3(0.0, 0.0, 1.0), normalize(normal), vec3(normalStrength));
    float foamAmount = 0.0;
    vec4 foamColor = vec4(1.0);
    if ((_289.paramIntData0.x & 4u) > 0u)
    {
        if ((_289.paramIntData0.x & 1u) > 0u)
        {
            foamAmount = 1.0 - min((waterDepth - _289.waterParams1.y) / _289.waterParams1.w, 1.0);
        }
        vec2 foamUV = ((uvFoam * _289.waterParams4.w) + (waterDUDV * _289.waterParams3.z)) + ((normal.xy * _289.waterParams2.z) * _289.waterParams3.z);
        if ((_289.paramIntData0.x & 2u) > 0u)
        {
            foamColor *= mix(SAMPLE_TEXTURE(waterFoamTex, foamUV + flowUV0), SAMPLE_TEXTURE(waterFoamTex, foamUV + flowUV1), vec4(flowPhaseBlendFactor));
            foamAmount = max(foamAmount, clamp(flowIntensity, 0.0, 1.0) * _289.waterParams2.x);
        }
        else
        {
            foamColor *= SAMPLE_TEXTURE(waterFoamTex, foamUV);
        }
        foamColor.w *= foamColor.w;
        foamAmount = (foamAmount * foamColor.w) + foamAmount;
        foamAmount = (foamAmount * _289.waterParams2.w) - (_289.waterParams2.w - 1.0);
        foamAmount = clamp(foamAmount, 0.0, 1.0);
        normal = mix(normal, vec3(0.0, 0.0, 1.0), vec3(foamAmount));
    }
    float debrisUVScale = _289.tint.w;
    vec2 debrisUV = (baseUV * debrisUVScale) + (normal.xy * _289.waterParams2.z);
    vec4 debrisColor;
    if ((_289.paramIntData0.x & 2u) > 0u)
    {
        debrisColor = mix(SAMPLE_TEXTURE(waterDebrisTex, debrisUV + flowUV0), SAMPLE_TEXTURE(waterDebrisTex, debrisUV + flowUV1), vec4(flowPhaseBlendFactor));
    }
    else
    {
        debrisColor = SAMPLE_TEXTURE(waterDebrisTex, debrisUV);
    }
    vec3 _838 = debrisColor.xyz * debrisTint.xyz;
    debrisColor = vec4(_838.x, _838.y, _838.z, debrisColor.w);
    vec3 _845 = foamColor.xyz * foamTint.xyz;
    foamColor = vec4(_845.x, _845.y, _845.z, foamColor.w);
    float foamAlpha = foamAmount * foamTint.w;
    float debrisAlpha = debrisColor.w * debrisTint.w;
    translucency = max(translucency, foamAlpha);
    translucency = max(translucency, debrisAlpha);
    flrOutput.albedo.w = softShoreAmount;
    vec3 _874 = mix(flrOutput.albedo.xyz, debrisColor.xyz, vec3(debrisAlpha));
    flrOutput.albedo = vec4(_874.x, _874.y, _874.z, flrOutput.albedo.w);
    vec3 _885 = mix(flrOutput.albedo.xyz, foamColor.xyz, vec3(foamAlpha));
    flrOutput.albedo = vec4(_885.x, _885.y, _885.z, flrOutput.albedo.w);
    flrOutput.normalsTS = normal;
    normal = normalize(((flrInput.tangent.xyz * normal.x) + (flrInput.binormal * normal.y)) + (flrInput.normal * normal.z));
    vec3 param_22 = normal;
    vec3 param_23 = flrInput.normal;
    flrOutput.normals = vec4(encodeNormal(param_22), encodeNormal(param_23));
    if ((_289.paramIntData0.x & 2u) > 0u)
    {
        vec3 _943 = mix(SAMPLE_TEXTURE(waterParameterTex, baseUV + flowUV0), SAMPLE_TEXTURE(waterParameterTex, baseUV + flowUV1), vec4(flowPhaseBlendFactor)).xyz;
        flrOutput.parameter = vec4(_943.x, _943.y, _943.z, flrOutput.parameter.w);
    }
    else
    {
        vec3 _951 = SAMPLE_TEXTURE(waterParameterTex, baseUV).xyz;
        flrOutput.parameter = vec4(_951.x, _951.y, _951.z, flrOutput.parameter.w);
    }
    flrOutput.parameter.x = min(flrOutput.parameter.x, 1.0 - debrisAlpha);
    flrOutput.parameter.y = min(flrOutput.parameter.y, 1.0 - foamAlpha);
    flrOutput.parameter.y = min(flrOutput.parameter.y, 1.0 - debrisAlpha);
    vec3 materialParameter = _289.waterParams4.xyz;
    flrOutput.parameter.x = clamp((flrOutput.parameter.x * clamp(materialParameter.x, 0.0, 1.0)) + clamp(materialParameter.x - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.y = clamp((flrOutput.parameter.y * clamp(materialParameter.y, 0.0, 1.0)) + clamp(materialParameter.y - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.z = 1.0 - clamp((1.0 - flrOutput.parameter.z) * materialParameter.z, 0.0, 1.0);
    flrOutput.parameter.w = translucency;
}

void main()
{
    FlrInput flrInput;
    flrInput.posWS = inPosWS;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal;
    flrInput.tangent = inTangent;
    flrInput.binormal = inBinormal;
    flrInput.pos = vec4(0.0);
    flrInput.dynamicResolutionFactor = vec2(1.0);
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outAlbedo = flrOutput.albedo;
    outNormals = flrOutput.normals;
    outParameter = flrOutput.parameter;
    outParameter.w = 0.0;
    outMotionVector = vec4(vec2(0.0).x, vec2(0.0).y, outMotionVector.z, outMotionVector.w);
    outMotionVector.z = float(_344.params0.w);
    outMotionVector.w = 1.0;
}

