#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct MaterialData
{
    uvec4 paramIntData0;
    vec4 paramFloatData0;
    vec4 paramFloatData1;
    vec4 paramFloatData2;
    vec4 paramFloatData3;
    vec4 paramFloatData4;
    vec4 paramFloatData5;
    vec4 paramFloatData6;
    vec4 paramFloatData7;
    vec4 paramFloatData8;
    vec4 paramFloatData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
};

struct LightingData
{
    vec3 L;
    vec2 energyDiffuseSpecular;
    vec3 posWS;
    vec3 posVS;
    vec3 cameraPosWS;
    vec3 V;
    vec3 N;
    vec3 NVtx;
    vec3 T;
    vec4 lightingWeights0;
    vec4 lightingWeights1;
    vec3 baseColor;
    float metalness;
    float specular;
    float roughness;
    float occlusion;
    float translucency;
    float ambientOcclusion;
    vec3 emissive;
    vec3 irradiance;
    vec4 materialBlendMask;
    uint materialId;
    MaterialData materialData;
    vec4 ssrSample;
    vec3 diffuseColor;
    vec3 specularColor;
    vec3 R;
    vec3 H;
    float NdL;
    float NdV;
    float NdH;
    float VdH;
    float LdH;
    float roughness2;
    float roughness4;
    vec3 posWSneighborX;
    vec3 posWSneighborY;
    float time;
    float depthFixup;
};

struct LightingDataDesc
{
    vec3 baseColor;
    float roughness;
    float metalness;
    float occlusion;
    float translucency;
    vec3 N;
    vec3 NVtx;
    float ambientOcclusion;
    vec3 emissive;
    vec3 irradiance;
    vec4 materialBlendMask;
    vec4 lightingWeights0;
    vec4 lightingWeights1;
    vec3 cameraPosWS;
    vec3 posWS;
    vec3 posVS;
    uint materialId;
    MaterialData materialData;
};

struct LightData
{
    vec4 posAndRadiusWS;
    vec4 colorAndIntensity;
    vec4 lightDirectionAndFalloffExp;
    vec4 lightTangent;
    uvec4 lightTypeAndFlags;
    vec4 spotLightAngles;
    vec4 sphereAndShadowParams;
};

struct ExtendedLightData
{
    mat4 boxInverseWorldMatrix;
    mat4 viewProjectionMatrix;
    uvec4 params0;
    vec4 projectorTransform;
};

struct FlrInput
{
    vec3 normal;
    vec3 binormal;
    vec4 tangent;
    vec2 uv0;
    vec4 color;
    vec4 pos;
    vec3 posWS;
    vec4 prevPos;
};

struct FlrOutput
{
    vec4 albedo;
    vec4 normals;
    vec4 parameter;
    uint materialId;
    uint miscParams0;
    vec2 motionVector;
    vec3 normalsTS;
};

struct SH9Compressed
{
    uvec4 data0[2];
    uvec4 data1;
};

struct TexturePatchData
{
    uvec4 index;
    vec4 transform;
    vec4 uvScaleOffset;
};

struct SH9Compressed_1
{
    uvec4 data0[2];
    uvec4 data1;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 0, binding = 32, std140) uniform PerView
{
    mat4 invViewProjMatrix;
    mat4 invProjMatrix;
    vec4 nearFar;
    vec4 nearFarWidthHeight;
    vec4 cameraPositionWS;
    vec4 dirLightDirWS;
    vec4 dirLightColorAndIntens;
    uvec4 dirLightProjectorParams;
    vec4 dirLightProjectorTransform;
    mat4 dirLightProjectorViewProjMatrix;
    SH9Compressed skyModelSH;
    SH9Compressed globalBackgroundSH;
    mat4 pssmViewProjMatrices[5];
    mat4 localLightViewProjMatrices[16];
    vec4 shadowParameters0;
    vec4 playerWorldPosition;
    vec4 lightingWeights0;
    vec4 lightingWeights1;
    vec4 dynamicResolutionParameters;
    vec4 frameParameters0;
    vec4 frameParameters1;
    vec4 projectionOffsets;
    vec4 irradianceGridWorldPosition;
    vec4 irradianceGridHalfExtents;
    uvec4 irradianceGridResolution;
    mat4 viewMatrix;
    vec4 volumetricsParams2;
} _2399;

layout(set = 0, binding = 41, std430) readonly buffer extendedLightBuffer
{
    ExtendedLightData _extendedLightBuffer[];
} _2799;

layout(set = 0, binding = 43, std430) readonly buffer resourceIndexBuffer
{
    uint _resourceIndexBuffer[];
} _3468;

layout(set = 0, binding = 40, std430) readonly buffer lightBuffer
{
    LightData _lightBuffer[];
} _3511;

layout(set = 0, binding = 31, std140) uniform PerMaterial
{
    uvec4 paramIntData0;
    vec4 tint;
    vec4 parameters0;
    vec4 uvTransform;
    vec4 paramData3;
    vec4 normalParams;
    vec4 paramData5;
    vec4 paramData6;
    vec4 paramData7;
    vec4 paramData8;
    vec4 paramData9;
    uvec4 paramImgIDs0;
    uvec4 paramImgIDs1;
    uvec4 paramImgIDs2;
} _3893;

layout(set = 0, binding = 30, std140) uniform PerInstance
{
    uvec4 params0;
    vec4 materialParams0;
    vec4 materialParams1;
    vec4 materialParams2;
    vec4 materialParams3;
} _3970;

layout(set = 0, binding = 8, std430) readonly buffer TexturePatchBuffer
{
    TexturePatchData _TexturePatchBuffer[];
} _4259;

layout(set = 0, binding = 44, std430) readonly buffer materialDataBuffer
{
    MaterialData _materialDataBuffer[];
} _4279;

layout(set = 0, binding = 45, std430) readonly buffer irradianceGridBuffer
{
    SH9Compressed_1 _irradianceGridBuffer[];
} _4819;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _4823;

layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 0, binding = 28) uniform sampler2DArrayShadow shadowBufferTex;
layout(set = 0, binding = 20) uniform SAMPLER_2D albedoTex;
layout(set = 0, binding = 26) uniform SAMPLER_2D dissolveTex;
layout(set = 0, binding = 22) uniform SAMPLER_2D parameterTex;
layout(set = 0, binding = 23) uniform SAMPLER_2D parameterOptional0Tex;
layout(set = 0, binding = 21) uniform SAMPLER_2D normalTex;
layout(set = 0, binding = 24) uniform SAMPLER_2D detailNormalTex;
layout(set = 0, binding = 27) uniform SAMPLER_2D colorCorrectionMaskTex;
layout(set = 0, binding = 29) uniform SAMPLER_2D irradianceTex;
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;
layout(set = 0, binding = 25) uniform SAMPLER_2D flowTex;

layout(location = 1) in vec2 inUV0;
layout(location = 0) in vec3 inNormal;
layout(location = 2) in vec4 inTangent;
layout(location = 3) in vec3 inBinormal;
layout(location = 4) in vec3 inPosWS;
layout(location = 6) in vec4 inPosition;
layout(location = 0) out vec4 outColor;

float _674;
float _691;
float _708;
bool _3203;

vec4 flrSampleBindless(sampler samplerState, vec2 uv, uint textureIdx)
{
    return texture(sampler2D(globalTextures[nonuniformEXT(textureIdx)], samplerState), uv);
}

vec3 decodeNormalMap(vec4 normalSample)
{
    vec2 _548 = (normalSample.wy - vec2(0.5)) * 2.0;
    vec3 normal;
    normal = vec3(_548.x, _548.y, normal.z);
    normal.z = sqrt(1.0 - ((normal.x * normal.x) - (normal.y * normal.y)));
    return normal;
}

vec4 flrSampleLevelBindless(sampler samplerState, vec2 uv, float lod, uint textureIdx)
{
    return textureLod(sampler2D(globalTextures[nonuniformEXT(textureIdx)], samplerState), uv, lod);
}

vec3 blendNormalMaps(vec3 base, vec3 blend)
{
    vec3 blendedNormal = normalize(vec3(base.xy + blend.xy, base.z * blend.z));
    return blendedNormal;
}

vec4 decodeRGBA8uint(uint colorU)
{
    return vec4(float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375, float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375);
}

vec3 gammaToLinear(vec3 color)
{
    return pow(clamp(color, vec3(0.0), vec3(1.0)), vec3(2.2000000476837158203125));
}

vec2 signNotZero(vec2 v)
{
    return vec2((v.x >= 0.0) ? 1.0 : (-1.0), (v.y >= 0.0) ? 1.0 : (-1.0));
}

vec2 encodeNormal(vec3 v)
{
    vec2 p = v.xy * (1.0 / ((abs(v.x) + abs(v.y)) + abs(v.z)));
    vec2 _490;
    if (v.z <= 0.0)
    {
        vec2 param = p;
        _490 = (vec2(1.0) - abs(p.yx)) * signNotZero(param);
    }
    else
    {
        _490 = p;
    }
    return _490;
}

float getLuminance(vec3 color)
{
    return dot(color, vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875));
}

vec4 decodeARGB8uint(uint colorU)
{
    return vec4(float((colorU & 16711680u) >> uint(16)) * 0.003921560011804103851318359375, float((colorU & 65280u) >> uint(8)) * 0.003921560011804103851318359375, float(colorU & 255u) * 0.003921560011804103851318359375, float((colorU & 4278190080u) >> uint(24)) * 0.003921560011804103851318359375);
}

vec2 getMotionVector(vec4 currPos, vec4 prevPos, vec4 projectionOffsets)
{
    vec2 currPosNDC = (currPos.xy / vec2(currPos.w)) + projectionOffsets.xy;
    vec2 prevPosNDC = (prevPos.xy / vec2(prevPos.w)) + projectionOffsets.zw;
    currPosNDC.y = 1.0 - currPosNDC.y;
    prevPosNDC.y = 1.0 - prevPosNDC.y;
    vec2 motionVector = (prevPosNDC - currPosNDC) * vec2(0.5, -0.5);
    motionVector = clamp(motionVector, vec2(-1.0), vec2(1.0));
    return motionVector;
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    vec2 uv = (flrInput.uv0 * _3893.uvTransform.xy) + _3893.uvTransform.zw;
    if ((_3893.paramIntData0.x & 8u) > 0u)
    {
        uv = (flrInput.uv0 * _3893.uvTransform.xy) + (_3893.uvTransform.zw * _2399.frameParameters0.x);
    }
    vec4 albedoValue = SAMPLE_TEXTURE1(albedoTex, uv, _2399.frameParameters1.x);
    if ((_3893.paramIntData0.x & 32u) > 0u)
    {
        float distToCameraNorm = pow(clamp(distance(_2399.cameraPositionWS.xyz, flrInput.posWS) / 200.0, 0.0, 1.0), 10.0);
        if (distToCameraNorm < 1.0)
        {
            float dissolveValue = SAMPLE_TEXTURE(dissolveTex, uv * _3893.paramData6.x).x;
            if (distToCameraNorm < dissolveValue)
            {
                discard;
            }
        }
    }
    uvec2 skinAlbedoNormalIDs = uvec2(floatBitsToUint(_3970.materialParams3.x) & 65535u, (floatBitsToUint(_3970.materialParams3.x) >> uint(16)) & 65535u);
    uvec2 skinParameterDetailNormalIDs = uvec2(floatBitsToUint(_3970.materialParams3.y) & 65535u, (floatBitsToUint(_3970.materialParams3.y) >> uint(16)) & 65535u);
    vec2 param = uv;
    uint param_1 = skinAlbedoNormalIDs.x;
    vec4 albedoOldValue = flrSampleBindless(linearRepeatSampler, param, param_1);
    albedoValue = mix(albedoValue, albedoOldValue, vec4(_3970.materialParams2.w));
    if (floatBitsToUint(_3970.materialParams1.x) == 501163121u)
    {
        float dissolveValue_1 = SAMPLE_TEXTURE(dissolveTex, uv * _3893.paramData6.x).x;
        vec3 _4033 = albedoValue.xyz + ((vec3(0.0, 0.5, 1.0) * float(abs(dissolveValue_1 - _3970.materialParams0.z) < 0.100000001490116119384765625)) * 1000.0);
        albedoValue = vec4(_4033.x, _4033.y, _4033.z, albedoValue.w);
        vec3 _4039 = min(albedoValue.xyz, vec3(8.0));
        albedoValue = vec4(_4039.x, _4039.y, _4039.z, albedoValue.w);
    }
    float alphaValue = 1.0;
    float alphaCutoff = 1.0;
    if (_3970.materialParams0.z < 1.0)
    {
        float dissolveValue_2 = SAMPLE_TEXTURE(dissolveTex, uv * _3893.paramData6.x).x;
        alphaCutoff = _3970.materialParams0.z;
        alphaValue = alphaCutoff - dissolveValue_2;
    }
    if ((_3893.paramIntData0.x & 1u) > 0u)
    {
        alphaCutoff *= (1.0 - _3893.parameters0.w);
        alphaCutoff = 1.0 - clamp(alphaCutoff, 0.0, 1.0);
        alphaValue *= albedoValue.w;
        alphaValue -= alphaCutoff;
    }
    if (alphaValue < 0.0)
    {
        discard;
    }
    float _4092;
    if ((_3893.paramIntData0.x & 128u) > 0u)
    {
        _4092 = 1.0 / max(albedoValue.w, 0.00999999977648258209228515625);
    }
    else
    {
        _4092 = 1.0;
    }
    vec3 _4103 = albedoValue.xyz * _4092;
    albedoValue = vec4(_4103.x, _4103.y, _4103.z, albedoValue.w);
    vec4 parameterValue = SAMPLE_TEXTURE1(parameterTex, uv, _2399.frameParameters1.x);
    vec2 param_2 = uv;
    uint param_3 = skinParameterDetailNormalIDs.x;
    vec4 parameterOldValue = flrSampleBindless(linearRepeatSampler, param_2, param_3);
    parameterValue = mix(parameterValue, parameterOldValue, vec4(_3970.materialParams2.w));
    vec4 parameterOptional0Value = SAMPLE_TEXTURE1(parameterOptional0Tex, uv, _2399.frameParameters1.x);
    float translucency = parameterOptional0Value.x;
    vec4 normalValue = SAMPLE_TEXTURE1(normalTex, uv, _3893.normalParams.y + _2399.frameParameters1.x);
    vec4 param_4 = normalValue;
    vec3 normal = decodeNormalMap(param_4);
    vec2 param_5 = uv;
    float param_6 = _3893.normalParams.y;
    uint param_7 = skinAlbedoNormalIDs.y;
    vec4 normalOldValue = flrSampleLevelBindless(linearRepeatSampler, param_5, param_6, param_7);
    vec4 param_8 = normalOldValue;
    vec3 normalOld = decodeNormalMap(param_8);
    normalOld = mix(vec3(0.0, 0.0, 1.0), normalOld, vec3(_3970.materialParams2.w));
    vec3 param_9 = normal;
    vec3 param_10 = normalOld;
    normal = blendNormalMaps(param_9, param_10);
    vec2 _4178 = normal.xy * _3893.normalParams.x;
    normal = vec3(_4178.x, _4178.y, normal.z);
    flrOutput.normalsTS = normal;
    if (_3893.normalParams.z > 0.0)
    {
        vec2 detailNormalUV = uv * _3893.normalParams.w;
        vec4 param_11 = SAMPLE_TEXTURE1(detailNormalTex, detailNormalUV, _2399.frameParameters1.x);
        vec3 detailNormal = decodeNormalMap(param_11);
        vec2 _4206 = detailNormal.xy * _3893.normalParams.z;
        detailNormal = vec3(_4206.x, _4206.y, detailNormal.z);
        detailNormal = normalize(detailNormal);
        vec3 param_12 = normal;
        vec3 param_13 = detailNormal;
        normal = blendNormalMaps(param_12, param_13);
        vec2 param_14 = detailNormalUV;
        uint param_15 = skinParameterDetailNormalIDs.y;
        vec4 param_16 = flrSampleBindless(linearRepeatSampler, param_14, param_15);
        vec3 detailNormalOld = decodeNormalMap(param_16);
        vec2 _4229 = detailNormalOld.xy * _3893.normalParams.z;
        detailNormalOld = vec3(_4229.x, _4229.y, detailNormalOld.z);
        detailNormalOld = normalize(detailNormalOld);
        detailNormalOld = mix(vec3(0.0, 0.0, 1.0), detailNormalOld, vec3(_3970.materialParams2.w));
        vec3 param_17 = normal;
        vec3 param_18 = detailNormalOld;
        normal = blendNormalMaps(param_17, param_18);
    }
    TexturePatchData data;
    MaterialData materialData;
    for (uint i = 0u; i < 4u; i++)
    {
        uint _4264 = floatBitsToUint(_3970.materialParams1.w) + i;
        data.index = _4259._TexturePatchBuffer[_4264].index;
        data.transform = _4259._TexturePatchBuffer[_4264].transform;
        data.uvScaleOffset = _4259._TexturePatchBuffer[_4264].uvScaleOffset;
        uint _4283 = clamp(data.index.x, 0u, 4096u);
        materialData.paramIntData0 = _4279._materialDataBuffer[_4283].paramIntData0;
        materialData.paramFloatData0 = _4279._materialDataBuffer[_4283].paramFloatData0;
        materialData.paramFloatData1 = _4279._materialDataBuffer[_4283].paramFloatData1;
        materialData.paramFloatData2 = _4279._materialDataBuffer[_4283].paramFloatData2;
        materialData.paramFloatData3 = _4279._materialDataBuffer[_4283].paramFloatData3;
        materialData.paramFloatData4 = _4279._materialDataBuffer[_4283].paramFloatData4;
        materialData.paramFloatData5 = _4279._materialDataBuffer[_4283].paramFloatData5;
        materialData.paramFloatData6 = _4279._materialDataBuffer[_4283].paramFloatData6;
        materialData.paramFloatData7 = _4279._materialDataBuffer[_4283].paramFloatData7;
        materialData.paramFloatData8 = _4279._materialDataBuffer[_4283].paramFloatData8;
        materialData.paramFloatData9 = _4279._materialDataBuffer[_4283].paramFloatData9;
        materialData.paramImgIDs0 = _4279._materialDataBuffer[_4283].paramImgIDs0;
        materialData.paramImgIDs1 = _4279._materialDataBuffer[_4283].paramImgIDs1;
        materialData.paramImgIDs2 = _4279._materialDataBuffer[_4283].paramImgIDs2;
        vec4 transform = data.transform;
        uvec4 textureIndices = materialData.paramImgIDs0;
        vec2 finalFeatureUv = flrInput.uv0;
        finalFeatureUv = (finalFeatureUv * transform.xy) - transform.zw;
        finalFeatureUv += vec2(0.5);
        vec2 featureUvOffset = abs(vec2(0.5) - finalFeatureUv);
        if (all(lessThanEqual(featureUvOffset, vec2(0.5))))
        {
            if (data.index.x != 0u)
            {
                vec2 param_19 = finalFeatureUv;
                uint param_20 = textureIndices.x;
                vec4 patchAlbedo = flrSampleBindless(linearRepeatSampler, param_19, param_20);
                vec2 param_21 = finalFeatureUv;
                uint param_22 = textureIndices.y;
                vec4 patchNormal = flrSampleBindless(linearRepeatSampler, param_21, param_22);
                vec2 param_23 = finalFeatureUv;
                uint param_24 = textureIndices.z;
                vec4 patchParameter = flrSampleBindless(linearRepeatSampler, param_23, param_24);
                finalFeatureUv = (finalFeatureUv * vec2(1.0)) + vec2(0.0);
                vec4 param_25 = patchNormal;
                vec3 patchNormalDecoded = decodeNormalMap(param_25);
                vec2 _4380 = patchNormalDecoded.xy * materialData.paramFloatData4.z;
                patchNormalDecoded = vec3(_4380.x, _4380.y, patchNormalDecoded.z);
                patchNormalDecoded = normalize(patchNormalDecoded);
                uint param_26 = floatBitsToUint(_3970.materialParams1.z);
                vec3 param_27 = decodeRGBA8uint(param_26).xyz;
                vec3 patchTint = gammaToLinear(param_27);
                vec3 _4397 = patchAlbedo.xyz * patchTint;
                patchAlbedo = vec4(_4397.x, _4397.y, _4397.z, patchAlbedo.w);
                albedoValue = mix(albedoValue, patchAlbedo, vec4(patchAlbedo.w));
                normal = mix(normal, patchNormalDecoded, vec3(patchAlbedo.w));
                parameterValue = mix(parameterValue, patchParameter, vec4(patchAlbedo.w));
            }
        }
    }
    normal = normalize(((flrInput.tangent.xyz * normal.x) + (flrInput.binormal * normal.y)) + (flrInput.normal * normal.z));
    flrOutput.albedo = vec4(albedoValue.xyz.x, albedoValue.xyz.y, albedoValue.xyz.z, flrOutput.albedo.w);
    flrOutput.albedo.w = albedoValue.w;
    vec3 param_28 = normal;
    vec3 param_29 = flrInput.normal;
    flrOutput.normals = vec4(encodeNormal(param_28), encodeNormal(param_29));
    flrOutput.materialId = _3970.params0.w;
    vec3 colorCorrectionValue = SAMPLE_TEXTURE(colorCorrectionMaskTex, uv).xyz;
    float correctionMask = clamp(colorCorrectionValue.x * flrInput.color.w, 0.0, 1.0);
    uint param_30 = floatBitsToUint(_3970.materialParams2.x);
    vec3 param_31 = decodeRGBA8uint(param_30).xyz;
    vec3 tintPerInstance = gammaToLinear(param_31);
    vec3 perInstanceColor = tintPerInstance * flrOutput.albedo.xyz;
    vec3 _4500 = mix(flrOutput.albedo.xyz, perInstanceColor, vec3(correctionMask)) * _3970.materialParams2.z;
    flrOutput.albedo = vec4(_4500.x, _4500.y, _4500.z, flrOutput.albedo.w);
    vec3 correctedAlbedo = flrInput.color.zyx * flrOutput.albedo.xyz;
    vec3 param_32 = correctedAlbedo;
    float lum = getLuminance(param_32);
    correctedAlbedo = vec3(lum) + ((correctedAlbedo - vec3(lum)) * _3893.tint.z);
    uint param_33 = floatBitsToUint(_3893.tint.x);
    vec3 param_34 = decodeARGB8uint(param_33).xyz;
    correctedAlbedo = (correctedAlbedo * gammaToLinear(param_34)) * _3893.tint.y;
    vec3 _4545 = mix(flrOutput.albedo.xyz, correctedAlbedo, vec3(correctionMask));
    flrOutput.albedo = vec4(_4545.x, _4545.y, _4545.z, flrOutput.albedo.w);
    vec3 parameterScale = _3893.parameters0.xyz;
    flrOutput.parameter = vec4(parameterValue.xyz, 1.0);
    flrOutput.parameter.x = clamp((flrOutput.parameter.x * clamp(parameterScale.x, 0.0, 1.0)) + clamp(parameterScale.x - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.y = clamp((flrOutput.parameter.y * clamp(parameterScale.y, 0.0, 1.0)) + clamp(parameterScale.y - 1.0, 0.0, 1.0), 0.0, 1.0);
    flrOutput.parameter.z = 1.0 - clamp((1.0 - flrOutput.parameter.z) * parameterScale.z, 0.0, 1.0);
    flrOutput.parameter.w = translucency;
    vec4 param_35 = flrInput.pos;
    vec4 param_36 = flrInput.prevPos;
    vec4 param_37 = _2399.projectionOffsets;
    flrOutput.motionVector = getMotionVector(param_35, param_36, param_37);
    flrOutput.miscParams0 = _3970.params0.x & 8191u;
    if ((_3893.paramIntData0.x & 1024u) > 0u)
    {
        flrOutput.miscParams0 |= 16384u;
    }
    flrOutput.miscParams0 |= 32768u;
}

vec3 decodeNormal(vec2 e)
{
    vec3 v = vec3(e, (1.0 - abs(e.x)) - abs(e.y));
    if (v.z < 0.0)
    {
        vec2 param = v.xy;
        vec2 _534 = (vec2(1.0) - abs(v.yx)) * signNotZero(param);
        v = vec3(_534.x, _534.y, v.z);
    }
    return normalize(v);
}

void initLightingData(inout LightingData d, LightingDataDesc desc)
{
    d.L = vec3(0.0);
    d.V = vec3(0.0, 0.0, 1.0);
    d.energyDiffuseSpecular = vec2(0.0);
    d.diffuseColor = vec3(0.0);
    d.specularColor = vec3(0.0);
    d.R = vec3(0.0);
    d.H = vec3(0.0);
    d.NdL = 0.0;
    d.NdV = 0.0;
    d.NdH = 0.0;
    d.VdH = 0.0;
    d.LdH = 0.0;
    d.roughness2 = 0.0;
    d.roughness4 = 0.0;
    d.ssrSample = vec4(0.0);
    d.baseColor = desc.baseColor;
    d.roughness = desc.roughness;
    d.metalness = desc.metalness;
    d.occlusion = desc.occlusion;
    d.translucency = desc.translucency;
    d.materialBlendMask = desc.materialBlendMask;
    d.materialId = desc.materialId;
    d.materialData = desc.materialData;
    d.specular = desc.materialData.paramFloatData8.x;
    d.lightingWeights0 = desc.lightingWeights0;
    d.lightingWeights1 = desc.lightingWeights1;
    d.cameraPosWS = desc.cameraPosWS;
    d.posWS = desc.posWS;
    d.posVS = desc.posVS;
    d.N = desc.N;
    d.NVtx = desc.NVtx;
    d.T = vec3(0.0);
    d.ambientOcclusion = desc.ambientOcclusion;
    d.emissive = desc.emissive;
    d.irradiance = desc.irradiance;
    d.posWSneighborX = vec3(0.0);
    d.posWSneighborY = vec3(0.0);
    d.time = 0.0;
    d.depthFixup = 1.0;
}

uint calcGridDepthIndex(float depthVS)
{
    return uint((24.0 * log(depthVS)) * 0.0819264352321624755859375);
}

float calcGridDepthSlice(uint depthSliceIdx)
{
    return pow(200000.0, float(depthSliceIdx) * 0.0416666679084300994873046875);
}

uvec3 clampGridPos(uvec3 gridPos)
{
    return clamp(gridPos, uvec3(0u), uvec3(15u, 7u, 23u));
}

uvec3 calcGridPosForViewPos(vec3 posVS, vec2 nearFar, vec4 nearFarWidthHeight)
{
    float param = -posVS.z;
    uint depthIdx = calcGridDepthIndex(param);
    uint param_1 = depthIdx + 1u;
    float gridEndDepth = calcGridDepthSlice(param_1);
    float rayPos = (gridEndDepth - nearFar.x) / (nearFar.y - nearFar.x);
    vec2 gridHalfWidthHeight = mix(nearFarWidthHeight.xy, nearFarWidthHeight.zw, vec2(rayPos)) * 0.5;
    vec2 localPos = posVS.xy / gridHalfWidthHeight;
    uvec3 gridPos = uvec3(uvec2(((localPos * 0.5) + vec2(0.5)) * vec2(16.0, 8.0)), depthIdx);
    uvec3 param_2 = gridPos;
    return clampGridPos(param_2);
}

uint calcClusterIndex(uvec3 gridPos)
{
    return uint(dot(vec3(gridPos), vec3(390.0, 6240.0, 49920.0))) / 2u;
}

vec3 calcAOMultiBounce(vec3 ao, vec3 albedo)
{
    vec3 a = (albedo * 2.040400028228759765625) - vec3(0.3323999941349029541015625);
    vec3 b = (albedo * (-4.79510021209716796875)) + vec3(0.6417000293731689453125);
    vec3 c = (albedo * 2.755199909210205078125) + vec3(0.69029998779296875);
    return max(ao, ((((ao * a) + b) * ao) + c) * ao);
}

vec3 calcAOMultiBounce(float ao, vec3 albedo)
{
    vec3 param = vec3(ao);
    vec3 param_1 = albedo;
    return calcAOMultiBounce(param, param_1);
}

void initLightingDataFromGBuffer(inout LightingData d)
{
    float adjRough = (d.roughness * 0.949999988079071044921875) + 0.0500000007450580596923828125;
    d.roughness2 = adjRough * adjRough;
    d.roughness4 = d.roughness2 * d.roughness2;
    d.metalness = mix(0.0, d.metalness, d.lightingWeights1.y);
    d.diffuseColor = d.baseColor - (d.baseColor * d.metalness);
    d.specularColor = mix(vec3(d.specular) * 0.07999999821186065673828125, d.baseColor, vec3(d.metalness));
}

void calculateLightingDataBase(inout LightingData d)
{
    d.V = normalize(d.cameraPosWS - d.posWS);
    d.NdV = clamp(abs(dot(d.N, d.V)) + 9.9999999392252902907785028219223e-09, 0.0, 1.0);
    d.R = reflect(-d.V, d.N);
    d.R = normalize(mix(d.N, d.R, vec3((1.0 - d.roughness2) * (sqrt(1.0 - d.roughness2) + d.roughness2))));
}

float calcExponentialFalloff(LightData ld, vec3 lightDistVec)
{
    vec3 nlv = lightDistVec / vec3(ld.posAndRadiusWS.w + 0.001000000047497451305389404296875);
    float d2 = dot(nlv, nlv);
    return pow(abs(1.0 - clamp(d2, 0.0, 1.0)), ld.lightDirectionAndFalloffExp.w);
}

float calcSpecularOcclusionFromAo(float ao, float NdV, float r)
{
    return clamp((pow(abs(NdV + ao), exp2(((-16.0) * r) - 1.0)) - 1.0) + ao, 0.0, 1.0);
}

vec4 calcPosLS(vec3 posWS, mat4 shadowViewProjMatrix)
{
    vec4 posLS = shadowViewProjMatrix * vec4(posWS, 1.0);
    vec3 _968 = posLS.xyz / posLS.www;
    posLS = vec4(_968.x, _968.y, _968.z, posLS.w);
    vec2 _975 = (posLS.xy * 0.5) + vec2(0.5);
    posLS = vec4(_975.x, _975.y, posLS.z, posLS.w);
    return posLS;
}

uint findBestFittingSplit(vec3 posWS, inout vec4 posLS)
{
    for (uint i = 0u; i < uint(_2399.shadowParameters0.x); i++)
    {
        vec3 param = posWS;
        mat4 param_1 = _2399.pssmViewProjMatrices[i];
        posLS = calcPosLS(param, param_1);
        bool _2416 = all(lessThanEqual(posLS.xyz, vec3(1.0)));
        bool _2423;
        if (_2416)
        {
            _2423 = all(greaterThanEqual(posLS.xyz, vec3(0.0)));
        }
        else
        {
            _2423 = _2416;
        }
        if (_2423)
        {
            return i;
        }
    }
    return 4294967295u;
}

vec2 calcBiasedShadowUV(vec3 posWS, vec3 normalVtx, vec3 L, mat4 projMatrix, float factor)
{
    float weight = max(1.0 - max(dot(normalVtx, L), 0.0), 0.1500000059604644775390625);
    vec3 offsetWS = (normalVtx * weight) * factor;
    vec3 offsetPosWS = posWS + offsetWS;
    vec4 offsetPosLS = projMatrix * vec4(offsetPosWS, 1.0);
    vec2 offsetUV = ((offsetPosLS.xy / vec2(offsetPosLS.w)) * 0.5) + vec2(0.5);
    return offsetUV;
}

float sampleShadowMap(vec2 baseUv, float u, float v, vec2 shadowMapSizeInv, float depth, uint shadowMapIdx, sampler2DArrayShadow shadowBufferTex_1)
{
    vec2 uv = baseUv + (vec2(u, v) * shadowMapSizeInv);
    vec4 _1000 = vec4(vec3(uv, float(shadowMapIdx)), depth);
    return texture(shadowBufferTex_1, vec4(_1000.xyz, _1000.w));
}

float pcf(vec3 shadowPos, uint shadowMapIdx, sampler2DArrayShadow shadowBufferTex_1)
{
    ivec3 size = textureSize(shadowBufferTex_1, 0);
    vec2 shadowMapSize;
    shadowMapSize.x = float(size.x);
    shadowMapSize.y = float(size.y);
    float elements = float(size.z);
    vec2 shadowMapSizeInv = vec2(1.0) / shadowMapSize;
    float lightDepth = shadowPos.z;
    vec2 uv = shadowPos.xy * shadowMapSize;
    vec2 baseUv;
    baseUv.x = floor(uv.x + 0.5);
    baseUv.y = floor(uv.y + 0.5);
    float s = (uv.x + 0.5) - baseUv.x;
    float t = (uv.y + 0.5) - baseUv.y;
    baseUv -= vec2(0.5);
    baseUv *= shadowMapSizeInv;
    float sum = 0.0;
    float uw0 = 3.0 - (2.0 * s);
    float uw1 = 1.0 + (2.0 * s);
    float u0 = ((2.0 - s) / uw0) - 1.0;
    float u1 = (s / uw1) + 1.0;
    float vw0 = 3.0 - (2.0 * t);
    float vw1 = 1.0 + (2.0 * t);
    float v0 = ((2.0 - t) / vw0) - 1.0;
    float v1 = (t / vw1) + 1.0;
    vec2 param = baseUv;
    float param_1 = u0;
    float param_2 = v0;
    vec2 param_3 = shadowMapSizeInv;
    float param_4 = lightDepth;
    uint param_5 = shadowMapIdx;
    sum += ((uw0 * vw0) * sampleShadowMap(param, param_1, param_2, param_3, param_4, param_5, shadowBufferTex_1));
    vec2 param_6 = baseUv;
    float param_7 = u1;
    float param_8 = v0;
    vec2 param_9 = shadowMapSizeInv;
    float param_10 = lightDepth;
    uint param_11 = shadowMapIdx;
    sum += ((uw1 * vw0) * sampleShadowMap(param_6, param_7, param_8, param_9, param_10, param_11, shadowBufferTex_1));
    vec2 param_12 = baseUv;
    float param_13 = u0;
    float param_14 = v1;
    vec2 param_15 = shadowMapSizeInv;
    float param_16 = lightDepth;
    uint param_17 = shadowMapIdx;
    sum += ((uw0 * vw1) * sampleShadowMap(param_12, param_13, param_14, param_15, param_16, param_17, shadowBufferTex_1));
    vec2 param_18 = baseUv;
    float param_19 = u1;
    float param_20 = v1;
    vec2 param_21 = shadowMapSizeInv;
    float param_22 = lightDepth;
    uint param_23 = shadowMapIdx;
    sum += ((uw1 * vw1) * sampleShadowMap(param_18, param_19, param_20, param_21, param_22, param_23, shadowBufferTex_1));
    return sum * 0.0625;
}

float calcFading(inout vec3 posLS)
{
    posLS = abs((posLS * 2.0) - vec3(1.0));
    float l = max(max(posLS.x, posLS.y), posLS.z);
    return smoothstep(0.89999997615814208984375, 1.0, l);
}

float calcDirShadowAttenuation(inout vec3 posWS, vec3 normalVtx, vec3 L, float shadowFactor)
{
    float shadowAttenuation = 1.0;
    posWS += (L * 6.0);
    vec3 param = posWS;
    vec4 param_1;
    uint _2608 = findBestFittingSplit(param, param_1);
    vec4 posLS = param_1;
    uint shadowMapIdx = _2608;
    if (shadowMapIdx != 4294967295u)
    {
        vec3 param_2 = posWS;
        vec3 param_3 = normalVtx;
        vec3 param_4 = L;
        mat4 param_5 = _2399.pssmViewProjMatrices[shadowMapIdx];
        float param_6 = 10.0 + ((10.0 * float(shadowMapIdx)) * 1.0);
        vec2 _2630 = calcBiasedShadowUV(param_2, param_3, param_4, param_5, param_6);
        posLS = vec4(_2630.x, _2630.y, posLS.z, posLS.w);
        vec3 param_7 = posLS.xyz;
        uint param_8 = shadowMapIdx;
        shadowAttenuation = pcf(param_7, param_8, shadowBufferTex);
        float shadowAttenuation2 = 1.0;
        vec3 param_9 = posLS.xyz;
        float _2645 = calcFading(param_9);
        float fadeSplits = _2645;
        uint nextShadowMapIdx = shadowMapIdx + 1u;
        if (nextShadowMapIdx < uint(_2399.shadowParameters0.x))
        {
            vec3 param_10 = posWS;
            mat4 param_11 = _2399.pssmViewProjMatrices[nextShadowMapIdx];
            vec4 tempPosLS = calcPosLS(param_10, param_11);
            vec3 param_12 = posWS;
            vec3 param_13 = normalVtx;
            vec3 param_14 = L;
            mat4 param_15 = _2399.pssmViewProjMatrices[nextShadowMapIdx];
            float param_16 = 10.0 + ((10.0 * float(shadowMapIdx)) * 1.0);
            vec2 _2680 = calcBiasedShadowUV(param_12, param_13, param_14, param_15, param_16);
            tempPosLS = vec4(_2680.x, _2680.y, tempPosLS.z, tempPosLS.w);
            vec3 param_17 = tempPosLS.xyz;
            uint param_18 = nextShadowMapIdx;
            shadowAttenuation2 = pcf(param_17, param_18, shadowBufferTex);
            shadowAttenuation = mix(shadowAttenuation, shadowAttenuation2, fadeSplits);
        }
        else
        {
            if (shadowMapIdx == (uint(_2399.shadowParameters0.x) - 1u))
            {
                vec3 param_19 = posWS;
                mat4 param_20 = _2399.pssmViewProjMatrices[shadowMapIdx];
                vec4 posLSLastSplit = calcPosLS(param_19, param_20);
                vec3 param_21 = posLSLastSplit.xyz;
                float _2714 = calcFading(param_21);
                float fadeGlobal = _2714;
                shadowAttenuation = mix(shadowAttenuation, 1.0, fadeGlobal);
            }
        }
    }
    return mix(1.0, shadowAttenuation, shadowFactor);
}

vec3 calcDirProjectorAttenuation(vec3 posWS)
{
    vec3 att = vec3(1.0);
    uint projImageId = _2399.dirLightProjectorParams.x;
    if (projImageId != 4294967295u)
    {
        vec3 posLS = (_2399.dirLightProjectorViewProjMatrix * vec4(posWS, 1.0)).xyw;
        vec2 _2502 = posLS.xy / vec2(posLS.z);
        posLS = vec3(_2502.x, _2502.y, posLS.z);
        vec2 _2511 = posLS.xy * _2399.dirLightProjectorTransform.xy;
        posLS = vec3(_2511.x, _2511.y, posLS.z);
        vec2 _2526 = posLS.xy + ((_2399.dirLightProjectorTransform.zw * _2399.frameParameters0.x) * _2399.dirLightProjectorTransform.xy);
        posLS = vec3(_2526.x, _2526.y, posLS.z);
        vec2 _2533 = (posLS.xy * 0.5) + vec2(0.5);
        posLS = vec3(_2533.x, _2533.y, posLS.z);
        float projectorFactor = uintBitsToFloat(_2399.dirLightProjectorParams.y);
        vec2 param = posLS.xy;
        float param_1 = 0.0;
        uint param_2 = projImageId;
        att *= max(mix(vec3(1.0), flrSampleLevelBindless(linearRepeatSampler, param, param_1, param_2).xyz, vec3(projectorFactor)), vec3(0.0));
    }
    return att;
}

float calcDistanceFieldShadowing(inout vec3 L, inout vec3 N, inout vec3 posWS, inout float rayLength, inout uint stepsMax, inout float k, out uint totalSteps)
{
    L = L;
    N = N;
    posWS = posWS;
    rayLength = rayLength;
    k = k;
    stepsMax = stepsMax;
    totalSteps = 0u;
    return 1.0;
}

float calcDistanceFieldShadowing(vec3 L, vec3 N, vec3 posWS, float rayLength)
{
    vec3 param = L;
    vec3 param_1 = N;
    vec3 param_2 = posWS;
    float param_3 = rayLength;
    uint param_4 = 32u;
    float param_5 = 10.0;
    uint param_6;
    float _2455 = calcDistanceFieldShadowing(param, param_1, param_2, param_3, param_4, param_5, param_6);
    uint totalSteps = param_6;
    return _2455;
}

void calculateLDisc(inout LightingData d, float discRadius)
{
    float discR = sin(discRadius);
    float discD = cos(discRadius);
    float DdR = dot(d.L, d.R);
    vec3 S = d.R - (d.L * DdR);
    vec3 _1324;
    if (DdR < discD)
    {
        _1324 = normalize((d.L * discD) + (normalize(S) * discR));
    }
    else
    {
        _1324 = d.R;
    }
    d.L = _1324;
}

void calculateLightingData(inout LightingData d)
{
    d.H = normalize(d.L + d.V);
    d.NdH = dot(d.N, d.H);
    d.VdH = dot(d.V, d.H);
    d.LdH = dot(d.L, d.H);
}

float calcMicroShadowing(float AO, float NdL, float opacity)
{
    float aperture = (2.0 * AO) * AO;
    float microshadow = clamp((NdL + aperture) - 1.0, 0.0, 1.0);
    return mix(1.0, microshadow, opacity);
}

float diffuseLambert()
{
    return 1.0;
}

vec3 calcDiffuse(LightingData d)
{
    return (((d.diffuseColor * (d.lightingWeights0.x * d.energyDiffuseSpecular.x)) * 0.3183098733425140380859375) * diffuseLambert()) * d.NdL;
}

vec3 calcDiffuseOcclusionDirectLighting(float ssao, vec3 albedo, float intens)
{
    float param = ssao;
    vec3 param_1 = albedo;
    return mix(vec3(1.0), calcAOMultiBounce(param, param_1), vec3(intens));
}

float D_GGX(float NdH, float roughness4)
{
    float denom = ((NdH * NdH) * (roughness4 - 1.0)) + 1.0;
    return roughness4 / ((3.1415927410125732421875 * denom) * denom);
}

float V_Schlick(float NdL, float NdV, float roughness2)
{
    float k = roughness2 * 0.5;
    float V = (NdV * (1.0 - k)) + k;
    float L = (NdL * (1.0 - k)) + k;
    return 0.25 / (V * L);
}

vec3 F_Schlick(vec3 specularColor, float VdH)
{
    float fc = pow(abs(1.0 - VdH), 5.0);
    return vec3(clamp(50.0 * specularColor.y, 0.0, 1.0) * fc) + (specularColor * (1.0 - fc));
}

vec3 calcSpecular(LightingData d, float r2, float r4)
{
    float param = d.NdH;
    float param_1 = r4;
    float D = D_GGX(param, param_1) * d.energyDiffuseSpecular.y;
    float param_2 = d.NdL;
    float param_3 = d.NdV;
    float param_4 = r2;
    float V = V_Schlick(param_2, param_3, param_4);
    vec3 param_5 = d.specularColor;
    float param_6 = d.VdH;
    vec3 F = F_Schlick(param_5, param_6);
    return (F * ((d.lightingWeights0.y * D) * V)) * d.NdL;
}

vec3 calcTranslucentLighting(vec3 N, vec3 L, vec3 V, float pixelTranslucency, float materialTranslucencyPower, float materialTranslucencyDistortion, vec3 materialTranslucencyColor)
{
    vec3 translucencyColor = materialTranslucencyColor * pixelTranslucency;
    vec3 sssLightVector = L + ((-N) * materialTranslucencyDistortion);
    float sssDot = exp2((clamp(dot(V, -sssLightVector), 0.0, 1.0) * materialTranslucencyPower) - materialTranslucencyPower);
    return translucencyColor * sssDot;
}

void applyTranslucentLighting(inout vec3 outLightingDiffuse, LightingData d)
{
    bool _1731 = (d.materialData.paramIntData0.x & 4u) > 0u;
    bool _1737;
    if (_1731)
    {
        _1737 = d.translucency > 0.0;
    }
    else
    {
        _1737 = _1731;
    }
    if (_1737)
    {
        float materialTranslucencyPower = d.materialData.paramFloatData7.x;
        float materialTranslucencyDistortion = d.materialData.paramFloatData7.y;
        float materialTranslucencyStrength = d.materialData.paramFloatData7.z;
        uint param = floatBitsToUint(d.materialData.paramFloatData7.w);
        vec3 param_1 = decodeARGB8uint(param).xyz;
        vec3 materialTranslucencyColor = gammaToLinear(param_1);
        vec3 param_2 = d.N;
        vec3 param_3 = d.L;
        vec3 param_4 = d.V;
        float param_5 = d.translucency;
        float param_6 = materialTranslucencyPower;
        float param_7 = materialTranslucencyDistortion;
        vec3 param_8 = materialTranslucencyColor;
        vec3 translucentLighting = calcTranslucentLighting(param_2, param_3, param_4, param_5, param_6, param_7, param_8);
        vec3 translucencyScalar = (((translucentLighting * materialTranslucencyStrength) * d.lightingWeights0.x) * d.energyDiffuseSpecular.x) * 0.3183098733425140380859375;
        outLightingDiffuse += translucencyScalar;
    }
}

void calcDirLightLighting(inout LightingData d, float energy, vec3 lightColor, vec3 L, inout vec3 outLighting0, inout vec3 outLighting1, vec4 diffuseSpecularWeights, vec3 shadowAttenuation)
{
    d.L = L;
    d.energyDiffuseSpecular = vec2(energy);
    LightingData param = d;
    float param_1 = 0.0670000016689300537109375;
    calculateLDisc(param, param_1);
    d = param;
    d.NdL = dot(d.N, d.L);
    LightingData param_2 = d;
    calculateLightingData(param_2);
    d = param_2;
    vec3 lightDiffuse = vec3(0.0);
    vec3 lightSpecular = vec3(0.0);
    if (d.NdL > 0.0)
    {
        float param_3 = d.occlusion;
        float param_4 = d.NdL;
        float param_5 = 0.75;
        vec3 attenuation = (shadowAttenuation * calcMicroShadowing(param_3, param_4, param_5)) * lightColor;
        LightingData param_6 = d;
        float param_7 = d.ambientOcclusion;
        vec3 param_8 = d.baseColor;
        float param_9 = 0.85000002384185791015625;
        lightDiffuse = (calcDiffuse(param_6) * attenuation) * calcDiffuseOcclusionDirectLighting(param_7, param_8, param_9);
        float r = max(d.roughness, 0.60000002384185791015625);
        float r2 = r * r;
        float r4 = r2 * r2;
        LightingData param_10 = d;
        float param_11 = r2;
        float param_12 = r4;
        lightSpecular = calcSpecular(param_10, param_11, param_12) * attenuation;
    }
    else
    {
        vec3 param_13 = lightDiffuse;
        LightingData param_14 = d;
        applyTranslucentLighting(param_13, param_14);
        lightDiffuse = param_13;
        lightDiffuse *= 0.100000001490116119384765625;
        lightDiffuse *= lightColor;
    }
    outLighting0 += ((lightDiffuse * diffuseSpecularWeights.x) + (lightSpecular * diffuseSpecularWeights.y));
    outLighting1 += ((lightDiffuse * diffuseSpecularWeights.z) + (lightSpecular * diffuseSpecularWeights.w));
}

vec3 calcLocalLightShadowAttenuation(LightData ld, inout vec3 posWS, vec3 shadowBiasNormal)
{
    vec3 L = normalize(ld.posAndRadiusWS.xyz - posWS);
    posWS += (L * 6.0);
    uint localShadowBufferIdx = floatBitsToUint(ld.sphereAndShadowParams.z);
    float shadowAtt = 1.0;
    if (localShadowBufferIdx != 255u)
    {
        uint shadowBufferIdx = floatBitsToUint(ld.sphereAndShadowParams.w);
        vec3 param = posWS;
        mat4 param_1 = _2399.localLightViewProjMatrices[localShadowBufferIdx];
        vec3 posLS = calcPosLS(param, param_1).xyz;
        vec3 param_2 = posWS;
        vec3 param_3 = shadowBiasNormal;
        vec3 param_4 = L;
        mat4 param_5 = _2399.localLightViewProjMatrices[localShadowBufferIdx];
        float param_6 = 2.0;
        vec2 _2767 = calcBiasedShadowUV(param_2, param_3, param_4, param_5, param_6);
        posLS = vec3(_2767.x, _2767.y, posLS.z);
        vec3 param_7 = posLS;
        uint param_8 = shadowBufferIdx;
        shadowAtt = pcf(param_7, param_8, shadowBufferTex);
    }
    float shadowFactor = ld.lightTangent.w;
    vec3 coloredAtt = mix(vec3(1.0), vec3(shadowAtt), vec3(shadowFactor));
    uint flags = (ld.lightTypeAndFlags.x >> 16u) & 65535u;
    if ((flags & 4u) > 0u)
    {
        uint _2807 = ld.lightTypeAndFlags.y + (floatBitsToUint(_2399.frameParameters0.w) * 128u);
        ExtendedLightData eld;
        eld.boxInverseWorldMatrix = _2799._extendedLightBuffer[_2807].boxInverseWorldMatrix;
        eld.viewProjectionMatrix = _2799._extendedLightBuffer[_2807].viewProjectionMatrix;
        eld.params0 = _2799._extendedLightBuffer[_2807].params0;
        eld.projectorTransform = _2799._extendedLightBuffer[_2807].projectorTransform;
        vec3 posLS_1 = (eld.viewProjectionMatrix * vec4(posWS, 1.0)).xyw;
        vec2 _2835 = posLS_1.xy / vec2(posLS_1.z);
        posLS_1 = vec3(_2835.x, _2835.y, posLS_1.z);
        vec2 _2843 = posLS_1.xy * eld.projectorTransform.xy;
        posLS_1 = vec3(_2843.x, _2843.y, posLS_1.z);
        vec2 _2858 = posLS_1.xy + ((eld.projectorTransform.zw * _2399.frameParameters0.x) * eld.projectorTransform.xy);
        posLS_1 = vec3(_2858.x, _2858.y, posLS_1.z);
        vec2 _2865 = (posLS_1.xy * 0.5) + vec2(0.5);
        posLS_1 = vec3(_2865.x, _2865.y, posLS_1.z);
        float projectorFactor = uintBitsToFloat(eld.params0.y);
        vec2 param_9 = posLS_1.xy;
        float param_10 = 0.0;
        uint param_11 = eld.params0.x;
        vec3 projAtt = mix(vec3(1.0), flrSampleLevelBindless(linearRepeatSampler, param_9, param_10, param_11).xyz, vec3(projectorFactor));
        coloredAtt *= projAtt;
    }
    return coloredAtt;
}

float smoothLine(float dist, float lineDist, float lineWidth)
{
    float lineVal = 1.0;
    lineVal *= smoothstep(lineDist - lineWidth, lineDist - (lineWidth * 0.5), dist);
    lineVal *= (1.0 - smoothstep(lineDist - (lineWidth * 0.5), lineDist, dist));
    return lineVal;
}

bool renderCustomPointLight(float dist, vec3 lightDistVec, LightData ld, LightingData d, float factor, inout vec3 outLighting)
{
    bool isEffectHeartbeatSensor = ld.lightTypeAndFlags.w == 3718345796u;
    float outerPerimeter = ld.posAndRadiusWS.w * 0.4000000059604644775390625;
    if (isEffectHeartbeatSensor && (dist > outerPerimeter))
    {
        return true;
    }
    else
    {
        if (isEffectHeartbeatSensor)
        {
            vec3 effectColor = ld.colorAndIntensity.xyz;
            vec3 color = vec3(0.0);
            float distFade = pow(1.0 - clamp(ld.posAndRadiusWS.w / 2600.0, 0.0, 1.0), 0.5) * pow(clamp((ld.posAndRadiusWS.w / 2600.0) - 0.20000000298023223876953125, 0.0, 1.0), 3.0);
            float param = dist;
            float param_1 = outerPerimeter;
            float param_2 = 5.0;
            outLighting += (((effectColor * (smoothLine(param, param_1, param_2) * 500.0)) * factor) * distFade);
            float innerPerimeter = ld.posAndRadiusWS.w * 0.100000001490116119384765625;
            float innerPerimeterRange = outerPerimeter - innerPerimeter;
            float param_3 = dist;
            float param_4 = innerPerimeter;
            float param_5 = 5.0;
            outLighting += (((effectColor * (smoothLine(param_3, param_4, param_5) * 500.0)) * factor) * distFade);
            if ((dist <= outerPerimeter) && (dist >= innerPerimeter))
            {
                float fade = pow(1.0 - min(abs(dist - (innerPerimeter + (innerPerimeterRange * 0.5))) / (innerPerimeterRange * 0.5), 1.0), 2.0) * distFade;
                fade *= pow(abs(dot(d.NVtx, vec3(0.0, 1.0, 0.0))), 5.0);
                float pulse = max(sin((dist * 0.00999999977648258209228515625) - (_2399.frameParameters0.x * 3.0)), 0.0) + 0.5;
                vec2 p = (d.posWS.xz * 0.02999999932944774627685546875) + ((normalize(lightDistVec).xz * pulse) * 0.25);
                vec4 hC = floor(vec4(p, p - vec2(0.5, 1.0)) / vec4(1.0, 1.73205077648162841796875, 1.0, 1.73205077648162841796875)) + vec4(0.5);
                vec4 h = vec4(p - (hC.xy * vec2(1.0, 1.73205077648162841796875)), p - ((hC.zw + vec2(0.5)) * vec2(1.0, 1.73205077648162841796875)));
                vec4 _3102;
                if (dot(h.xy, h.xy) < dot(h.zw, h.zw))
                {
                    _3102 = vec4(h.xy, hC.xy);
                }
                else
                {
                    _3102 = vec4(h.zw, hC.zw + vec2(9.729999542236328125));
                }
                vec4 hex = _3102;
                float hexRand = fract(sin(dot(hex.zw, vec2(141.1300048828125, 289.970001220703125))) * 43758.546875);
                float hexRandAnimated = (sin((hexRand * 6.28299999237060546875) + (_2399.frameParameters0.x * 5.0)) * 0.5) + 0.5;
                vec2 p0 = abs(hex.xy);
                float hexEdgeDist = max(dot(p0, vec2(0.5, 0.866025388240814208984375)), p0.x);
                float param_6 = hexEdgeDist;
                float param_7 = 0.5;
                float param_8 = 0.0500000007450580596923828125;
                color += ((effectColor * 100.0) * smoothLine(param_6, param_7, param_8));
                float param_9 = hexEdgeDist;
                float param_10 = 0.4000000059604644775390625;
                float param_11 = 0.0500000007450580596923828125;
                color += (vec3(1.0) * smoothLine(param_9, param_10, param_11));
                color += ((effectColor * 5.0) * (1.0 - hexEdgeDist));
                outLighting = mix(outLighting, color, vec3(((fade * pulse) * hexRandAnimated) * factor));
            }
            return true;
        }
        else
        {
            return false;
        }
    }
}

float calcDist2(vec3 lightDistVec, float biasSqr)
{
    return dot(lightDistVec, lightDistVec) + biasSqr;
}

float square(float x)
{
    return x * x;
}

float calcInverseSqrRadiusCutoff(float dist2, float radius)
{
    float rcA = 1.0 / radius;
    float param = rcA;
    float rcB = square(param) * dist2;
    float param_1 = rcB;
    float rcC = clamp(1.0 - square(param_1), 0.0, 1.0);
    float param_2 = rcC;
    return square(param_2);
}

float calcInverseSqrFalloff(inout float dist2)
{
    dist2 *= 9.9999997473787516355514526367188e-05;
    return 1.0 / (dist2 + 1.0);
}

float calculateAreaLightFalloff(vec3 lightDistVec, LightData ld, inout float NdL, vec3 N, float biasSqr)
{
    vec3 param = lightDistVec;
    float param_1 = biasSqr;
    float dist2 = calcDist2(param, param_1);
    float param_2 = dist2;
    float param_3 = ld.posAndRadiusWS.w;
    float radialCutoff = calcInverseSqrRadiusCutoff(param_2, param_3);
    float radialFalloff = 1.0;
    if (ld.sphereAndShadowParams.y > 9.9999999392252902907785028219223e-09)
    {
        vec3 L0 = ld.lightTangent.xyz * ld.sphereAndShadowParams.y;
        vec3 L1 = lightDistVec - (L0 * 0.5);
        vec3 L2 = lightDistVec + (L0 * 0.5);
        float L1Sqr = dot(L1, L1) + biasSqr;
        float L2Sqr = dot(L2, L2) + biasSqr;
        float L1RSqr = inversesqrt(L1Sqr);
        float L2RSqr = inversesqrt(L2Sqr);
        float L1Len = L1Sqr * L1RSqr;
        float L2Len = L2Sqr * L2RSqr;
        radialFalloff = 10000.0 / (((L1Len * L2Len) + dot(L1, L2)) * 0.5);
        NdL = clamp(0.5 * ((dot(N, L1) * L1RSqr) + (dot(N, L2) * L2RSqr)), 0.0, 1.0);
    }
    else
    {
        float param_4 = dist2;
        float _1623 = calcInverseSqrFalloff(param_4);
        radialFalloff = _1623;
    }
    return radialCutoff * radialFalloff;
}

void conserveEnergyTube(inout float energy, float radius, float roughness4, float NdV, float invDistToLight2)
{
    float param = radius;
    float a2 = clamp(square(param) * invDistToLight2, 0.0, 1.0);
    energy *= sqrt(roughness4 / (roughness4 + (a2 / ((NdV * 3.599999904632568359375) + 0.4000000059604644775390625))));
}

void conserveEnergySphere(inout float energy, float radius, float roughness4, float NdV, float invDistToLight2)
{
    float param = radius;
    float a2 = clamp(square(param) * invDistToLight2, 0.0, 1.0);
    energy *= (roughness4 / (roughness4 + (a2 / ((NdV * 3.599999904632568359375) + 0.4000000059604644775390625))));
}

void calculateAreaLightL(inout LightingData d, inout float sphereRadius, float tubeLength, vec3 lightTangent, inout vec3 lightDistVec)
{
    float sphereLightHack = smoothstep(0.0, 0.100000001490116119384765625, d.roughness);
    sphereRadius = mix(1.0, sphereRadius, sphereLightHack * sphereLightHack);
    float invDistToLight2 = 1.0 / dot(lightDistVec, lightDistVec);
    if (tubeLength > 9.9999999392252902907785028219223e-09)
    {
        float param = d.energyDiffuseSpecular.y;
        float param_1 = tubeLength * 0.5;
        float param_2 = d.roughness4;
        float param_3 = d.NdV;
        float param_4 = invDistToLight2;
        conserveEnergyTube(param, param_1, param_2, param_3, param_4);
        d.energyDiffuseSpecular.y = param;
        vec3 L0 = lightTangent * tubeLength;
        vec3 L1 = lightDistVec - (L0 * 0.5);
        float param_5 = tubeLength;
        float len2 = square(param_5);
        float b = dot(d.R, L0);
        float param_6 = b;
        float b2 = square(param_6);
        float t = clamp(dot(L1, (d.R * b) - L0) / (len2 - b2), 0.0, 1.0);
        lightDistVec = L1 + (L0 * t);
    }
    float param_7 = d.energyDiffuseSpecular.y;
    float param_8 = sphereRadius;
    float param_9 = d.roughness4;
    float param_10 = d.NdV;
    float param_11 = invDistToLight2;
    conserveEnergySphere(param_7, param_8, param_9, param_10, param_11);
    d.energyDiffuseSpecular.y = param_7;
    vec3 closestPointRay = d.R * dot(lightDistVec, d.R);
    vec3 centerToRay = closestPointRay - lightDistVec;
    lightDistVec += (centerToRay * clamp(sphereRadius * inversesqrt(dot(centerToRay, centerToRay)), 0.0, 1.0));
    d.L = normalize(lightDistVec);
}

vec3 calcSpecular(LightingData d)
{
    LightingData param = d;
    float param_1 = d.roughness2;
    float param_2 = d.roughness4;
    return calcSpecular(param, param_1, param_2);
}

void calcLighting(LightingData d, out vec3 diffuse, out vec3 specular)
{
    LightingData param = d;
    diffuse = calcDiffuse(param);
    LightingData param_1 = d;
    specular = calcSpecular(param_1);
}

void calcPointLightLighting(LightData ld, inout LightingData d, inout vec3 outLighting0, inout vec3 outLighting1, vec4 diffuseSpecularWeights, vec3 shadowAttenuation)
{
    vec3 lightDistVec = ld.posAndRadiusWS.xyz - d.posWS;
    d.L = normalize(lightDistVec);
    d.NdL = dot(d.N, d.L);
    float sphereRadius = ld.sphereAndShadowParams.x;
    float tubeLength = ld.sphereAndShadowParams.y;
    vec3 attenuation = vec3(1.0);
    if ((((ld.lightTypeAndFlags.x >> 16u) & 65535u) & 1u) == 0u)
    {
        vec3 param = lightDistVec;
        LightData param_1 = ld;
        float param_2 = d.NdL;
        vec3 param_3 = d.N;
        float param_4 = 0.0;
        float _2094 = calculateAreaLightFalloff(param, param_1, param_2, param_3, param_4);
        d.NdL = param_2;
        attenuation = vec3(_2094);
    }
    else
    {
        LightData param_5 = ld;
        vec3 param_6 = lightDistVec;
        attenuation = vec3(calcExponentialFalloff(param_5, param_6));
    }
    attenuation *= ld.colorAndIntensity.xyz;
    if (any(greaterThan(attenuation, vec3(9.9999999392252902907785028219223e-09))))
    {
        d.energyDiffuseSpecular = ld.colorAndIntensity.ww;
        LightingData param_7 = d;
        float param_8 = sphereRadius;
        float param_9 = tubeLength;
        vec3 param_10 = ld.lightTangent.xyz;
        vec3 param_11 = lightDistVec;
        calculateAreaLightL(param_7, param_8, param_9, param_10, param_11);
        d = param_7;
        lightDistVec = param_11;
        LightingData param_12 = d;
        calculateLightingData(param_12);
        d = param_12;
        vec3 lightDiffuse = vec3(0.0);
        vec3 lightSpecular = vec3(0.0);
        if (d.NdL > 0.0)
        {
            LightingData param_13 = d;
            vec3 param_14;
            vec3 param_15;
            calcLighting(param_13, param_14, param_15);
            lightDiffuse = param_14;
            lightSpecular = param_15;
            float param_16 = d.ambientOcclusion;
            vec3 param_17 = d.baseColor;
            float param_18 = 0.85000002384185791015625;
            lightDiffuse *= (shadowAttenuation * calcDiffuseOcclusionDirectLighting(param_16, param_17, param_18));
            lightSpecular *= shadowAttenuation;
        }
        else
        {
            vec3 param_19 = lightDiffuse;
            LightingData param_20 = d;
            applyTranslucentLighting(param_19, param_20);
            lightDiffuse = param_19;
            lightDiffuse *= 0.100000001490116119384765625;
        }
        lightDiffuse *= attenuation;
        lightSpecular *= attenuation;
        outLighting0 += ((lightDiffuse * diffuseSpecularWeights.x) + (lightSpecular * diffuseSpecularWeights.y));
        outLighting1 += ((lightDiffuse * diffuseSpecularWeights.z) + (lightSpecular * diffuseSpecularWeights.w));
    }
}

float calcSpotLightAttenuation(LightData ld, vec3 L)
{
    float att = clamp((dot(L, -ld.lightDirectionAndFalloffExp.xyz) - ld.spotLightAngles.x) * ld.spotLightAngles.y, 0.0, 1.0);
    float param = att;
    return square(param);
}

void calcSpotLightLighting(LightData ld, inout LightingData d, inout vec3 outLighting0, inout vec3 outLighting1, vec4 diffuseSpecularWeights, vec3 factor)
{
    LightData param = ld;
    vec3 param_1 = d.L;
    vec3 spotAtt = factor * calcSpotLightAttenuation(param, param_1);
    if (any(greaterThan(spotAtt, vec3(9.9999999392252902907785028219223e-09))))
    {
        LightData param_2 = ld;
        LightingData param_3 = d;
        vec3 param_4 = outLighting0;
        vec3 param_5 = outLighting1;
        vec4 param_6 = diffuseSpecularWeights;
        vec3 param_7 = spotAtt;
        calcPointLightLighting(param_2, param_3, param_4, param_5, param_6, param_7);
        d = param_3;
        outLighting0 = param_4;
        outLighting1 = param_5;
    }
}

float calcExponentialFalloff(LightData ld, float _distance)
{
    float d2 = _distance * _distance;
    return pow(abs(1.0 - clamp(d2, 0.0, 1.0)), ld.lightDirectionAndFalloffExp.w);
}

float calcBoxLightAttenuation(vec3 posWS, LightData ld, ExtendedLightData eld)
{
    vec3 posBS = (eld.boxInverseWorldMatrix * vec4(posWS, 1.0)).xyz;
    float ret = 0.0;
    bool _2249 = all(lessThanEqual(posBS, vec3(1.0)));
    bool _2256;
    if (_2249)
    {
        _2256 = all(greaterThanEqual(posBS, vec3(-1.0)));
    }
    else
    {
        _2256 = _2249;
    }
    if (_2256)
    {
        vec2 a = vec2(1.0) - (clamp(abs(posBS.xz) - vec2(0.800000011920928955078125), vec2(0.0), vec2(1.0)) / vec2(0.20000000298023223876953125));
        ret = min(a.x, a.y);
        LightData param = ld;
        float param_1 = (posBS.y * 0.5) + 0.5;
        ret *= calcExponentialFalloff(param, param_1);
    }
    return ret;
}

void calcLocalLightLighting(LightData ld, inout LightingData d, inout vec3 outLighting0, inout vec3 outLighting1, vec4 diffuseSpecularWeights, inout vec3 factor)
{
    vec3 lightDistVec = ld.posAndRadiusWS.xyz - d.posWS;
    float dist = length(lightDistVec);
    d.L = lightDistVec / vec3(dist);
    uint lightType = ld.lightTypeAndFlags.x & 65535u;
    if ((lightType == 2u) || (lightType == 1u))
    {
        LightData param = ld;
        vec3 param_1 = d.posWS;
        vec3 param_2 = d.NVtx;
        vec3 _3238 = calcLocalLightShadowAttenuation(param, param_1, param_2);
        factor *= _3238;
    }
    vec3 param_3 = d.L;
    vec3 param_4 = d.NVtx;
    vec3 param_5 = d.posWS;
    float param_6 = dist;
    factor *= calcDistanceFieldShadowing(param_3, param_4, param_5, param_6);
    vec3 lightingDiffuse = vec3(0.0);
    vec3 lightingSpecular = vec3(0.0);
    if (lightType == 0u)
    {
        float param_7 = dist;
        vec3 param_8 = lightDistVec;
        LightData param_9 = ld;
        LightingData param_10 = d;
        float param_11 = 1.0;
        vec3 param_12 = outLighting0;
        bool _3272 = renderCustomPointLight(param_7, param_8, param_9, param_10, param_11, param_12);
        outLighting0 = param_12;
        if (_3272)
        {
            return;
        }
        LightData param_13 = ld;
        LightingData param_14 = d;
        vec3 param_15 = lightingDiffuse;
        vec3 param_16 = lightingSpecular;
        vec4 param_17 = diffuseSpecularWeights;
        vec3 param_18 = factor;
        calcPointLightLighting(param_13, param_14, param_15, param_16, param_17, param_18);
        d = param_14;
        lightingDiffuse = param_15;
        lightingSpecular = param_16;
        outLighting0 += lightingDiffuse;
        outLighting1 += lightingSpecular;
    }
    else
    {
        if (lightType == 1u)
        {
            float cutoff = clamp(max(dist - ld.spotLightAngles.w, 0.0) / ld.spotLightAngles.w, 0.0, 1.0);
            LightData param_19 = ld;
            LightingData param_20 = d;
            vec3 param_21 = lightingDiffuse;
            vec3 param_22 = lightingSpecular;
            vec4 param_23 = diffuseSpecularWeights;
            vec3 param_24 = factor * cutoff;
            calcSpotLightLighting(param_19, param_20, param_21, param_22, param_23, param_24);
            d = param_20;
            lightingDiffuse = param_21;
            lightingSpecular = param_22;
            outLighting0 += lightingDiffuse;
            outLighting1 += lightingSpecular;
        }
        else
        {
            if (lightType == 2u)
            {
                uint _3350 = ld.lightTypeAndFlags.y + (floatBitsToUint(_2399.frameParameters0.w) * 128u);
                ExtendedLightData eld;
                eld.boxInverseWorldMatrix = _2799._extendedLightBuffer[_3350].boxInverseWorldMatrix;
                eld.viewProjectionMatrix = _2799._extendedLightBuffer[_3350].viewProjectionMatrix;
                eld.params0 = _2799._extendedLightBuffer[_3350].params0;
                eld.projectorTransform = _2799._extendedLightBuffer[_3350].projectorTransform;
                vec3 param_25 = d.posWS;
                LightData param_26 = ld;
                ExtendedLightData param_27 = eld;
                vec3 att = factor * calcBoxLightAttenuation(param_25, param_26, param_27);
                if (any(greaterThan(att, vec3(9.9999999392252902907785028219223e-09))))
                {
                    LightingData param_28 = d;
                    float param_29 = ld.colorAndIntensity.w;
                    vec3 param_30 = ld.colorAndIntensity.xyz;
                    vec3 param_31 = -ld.lightDirectionAndFalloffExp.xyz;
                    vec3 param_32 = outLighting0;
                    vec3 param_33 = outLighting1;
                    vec4 param_34 = diffuseSpecularWeights;
                    vec3 param_35 = att;
                    calcDirLightLighting(param_28, param_29, param_30, param_31, param_32, param_33, param_34, param_35);
                    d = param_28;
                    outLighting0 = param_32;
                    outLighting1 = param_33;
                }
            }
        }
    }
}

float colormapRed(float x)
{
    if (x < 0.75)
    {
        return (1012.0 * x) - 389.0;
    }
    else
    {
        return ((-1113.2276611328125) * x) + 1244.6119384765625;
    }
}

float colormapGreen(float x)
{
    if (x < 0.5)
    {
        return (1012.0 * x) - 129.0;
    }
    else
    {
        return ((-1012.0) * x) + 899.0;
    }
}

float colormapBlue(float x)
{
    if (x < 0.25)
    {
        return (1012.0 * x) + 131.0;
    }
    else
    {
        return ((-1012.0) * x) + 643.0;
    }
}

vec4 colormap(float x)
{
    float param = x;
    float r = clamp(colormapRed(param) / 255.0, 0.0, 1.0);
    float param_1 = x;
    float g = clamp(colormapGreen(param_1) / 255.0, 0.0, 1.0);
    float param_2 = x;
    float b = clamp(colormapBlue(param_2) / 255.0, 0.0, 1.0);
    return vec4(r, g, b, 1.0);
}

void calcLightingAll(inout LightingData d, inout vec3 outLighting0, inout vec3 outLighting1)
{
    vec3 param = d.posVS;
    vec2 param_1 = _2399.nearFar.xy;
    vec4 param_2 = _2399.nearFarWidthHeight;
    uvec3 clusterGridPos = calcGridPosForViewPos(param, param_1, param_2);
    uvec3 param_3 = clusterGridPos;
    uint clusterIdxNonUniform = calcClusterIndex(param_3);
    vec3 irrad = d.irradiance;
    float ao = min(d.occlusion, d.ambientOcclusion);
    float param_4 = ao;
    vec3 param_5 = d.baseColor;
    irrad *= calcAOMultiBounce(param_4, param_5);
    d.baseColor = mix(vec3(0.5), d.baseColor, vec3(d.lightingWeights1.z));
    LightingData param_6 = d;
    initLightingDataFromGBuffer(param_6);
    d = param_6;
    LightingData param_7 = d;
    calculateLightingDataBase(param_7);
    d = param_7;
    irrad *= ((d.diffuseColor * 0.3183098733425140380859375) * _2399.lightingWeights0.z);
    vec3 specular = vec3(0.0);
    vec3 specularIrrad = vec3(0.0);
    uint clusterIdx = clusterIdxNonUniform;
    uint lightCount = (_3468._resourceIndexBuffer[(clusterIdx + 2u) + (floatBitsToUint(_2399.frameParameters0.w) * 599040u)] >> uint(16)) & 65535u;
    LightData ld;
    vec3 _3595;
    vec3 _3601;
    for (uint i = 0u; i < lightCount; i++)
    {
        uint lightIndex = (_3468._resourceIndexBuffer[(((clusterIdx + (i * 3u)) + 3u) + 2u) + (floatBitsToUint(_2399.frameParameters0.w) * 599040u)] >> uint(16)) & 65535u;
        uint _3517 = lightIndex + (floatBitsToUint(_2399.frameParameters0.w) * 128u);
        ld.posAndRadiusWS = _3511._lightBuffer[_3517].posAndRadiusWS;
        ld.colorAndIntensity = _3511._lightBuffer[_3517].colorAndIntensity;
        ld.lightDirectionAndFalloffExp = _3511._lightBuffer[_3517].lightDirectionAndFalloffExp;
        ld.lightTangent = _3511._lightBuffer[_3517].lightTangent;
        ld.lightTypeAndFlags = _3511._lightBuffer[_3517].lightTypeAndFlags;
        ld.spotLightAngles = _3511._lightBuffer[_3517].spotLightAngles;
        ld.sphereAndShadowParams = _3511._lightBuffer[_3517].sphereAndShadowParams;
        uint lightType = ld.lightTypeAndFlags.x & 65535u;
        float falloffStartDistance = uintBitsToFloat(ld.lightTypeAndFlags.z);
        vec3 lightDistVec = ld.posAndRadiusWS.xyz - d.posWS;
        float distToLight = length(lightDistVec);
        vec3 lightVec = lightDistVec / vec3(distToLight);
        lightDistVec = lightVec * max(distToLight - falloffStartDistance, 0.0);
        ld.posAndRadiusWS.w -= falloffStartDistance;
        bool applyToAll = lightType == 5u;
        LightData param_8 = ld;
        vec3 param_9 = lightDistVec;
        float falloff = calcExponentialFalloff(param_8, param_9);
        vec3 _mod = mix(vec3(1.0), ld.colorAndIntensity.xyz * ld.colorAndIntensity.w, vec3(falloff * d.lightingWeights1.x));
        if (applyToAll || (lightType == 3u))
        {
            if (ld.colorAndIntensity.w < 1.0)
            {
                vec3 param_10 = _mod;
                vec3 param_11 = d.baseColor;
                _3601 = calcAOMultiBounce(param_10, param_11);
            }
            else
            {
                _3601 = _mod;
            }
            _3595 = _3601;
        }
        else
        {
            _3595 = vec3(1.0);
        }
        irrad *= _3595;
        specular *= mix(vec3(1.0), _mod, bvec3(applyToAll || (lightType == 4u)));
    }
    float param_12 = d.ambientOcclusion;
    float param_13 = d.NdV;
    float param_14 = d.roughness;
    specular *= calcSpecularOcclusionFromAo(param_12, param_13, param_14);
    specular *= _2399.lightingWeights0.w;
    outLighting0 = (irrad * 1.0) + (specular * 0.0);
    outLighting1 = (irrad * 0.0) + (specular * 0.0);
    if (_2399.dirLightColorAndIntens.w > 9.9999999392252902907785028219223e-09)
    {
        vec3 shadowBiasNormal = d.NVtx;
        float shadowFactor = _2399.dirLightDirWS.w;
        vec3 param_15 = d.posWS;
        vec3 param_16 = shadowBiasNormal;
        vec3 param_17 = _2399.dirLightDirWS.xyz;
        float param_18 = shadowFactor;
        float _3677 = calcDirShadowAttenuation(param_15, param_16, param_17, param_18);
        vec3 param_19 = d.posWS;
        vec3 dirLightAttenuation = vec3(_3677) * calcDirProjectorAttenuation(param_19);
        vec3 param_20 = _2399.dirLightDirWS.xyz;
        vec3 param_21 = d.NVtx;
        vec3 param_22 = d.posWS;
        float param_23 = 50000.0;
        dirLightAttenuation *= calcDistanceFieldShadowing(param_20, param_21, param_22, param_23);
        if (any(greaterThan(dirLightAttenuation, vec3(9.9999999392252902907785028219223e-09))))
        {
            LightingData param_24 = d;
            float param_25 = _2399.dirLightColorAndIntens.w;
            vec3 param_26 = _2399.dirLightColorAndIntens.xyz;
            vec3 param_27 = _2399.dirLightDirWS.xyz;
            vec3 param_28 = outLighting0;
            vec3 param_29 = outLighting1;
            vec4 param_30 = vec4(1.0, 0.0, 0.0, 0.0);
            vec3 param_31 = dirLightAttenuation;
            calcDirLightLighting(param_24, param_25, param_26, param_27, param_28, param_29, param_30, param_31);
            d = param_24;
            outLighting0 = param_28;
            outLighting1 = param_29;
        }
    }
    uint clusterIdx_1 = clusterIdxNonUniform;
    uint lightCount_1 = _3468._resourceIndexBuffer[clusterIdx_1 + (floatBitsToUint(_2399.frameParameters0.w) * 599040u)] & 65535u;
    LightData ld_1;
    for (uint i_1 = 0u; i_1 < lightCount_1; i_1++)
    {
        uint lightIndex_1 = _3468._resourceIndexBuffer[((clusterIdx_1 + (i_1 * 3u)) + 3u) + (floatBitsToUint(_2399.frameParameters0.w) * 599040u)] & 65535u;
        uint _3770 = lightIndex_1 + (floatBitsToUint(_2399.frameParameters0.w) * 128u);
        ld_1.posAndRadiusWS = _3511._lightBuffer[_3770].posAndRadiusWS;
        ld_1.colorAndIntensity = _3511._lightBuffer[_3770].colorAndIntensity;
        ld_1.lightDirectionAndFalloffExp = _3511._lightBuffer[_3770].lightDirectionAndFalloffExp;
        ld_1.lightTangent = _3511._lightBuffer[_3770].lightTangent;
        ld_1.lightTypeAndFlags = _3511._lightBuffer[_3770].lightTypeAndFlags;
        ld_1.spotLightAngles = _3511._lightBuffer[_3770].spotLightAngles;
        ld_1.sphereAndShadowParams = _3511._lightBuffer[_3770].sphereAndShadowParams;
        LightData param_32 = ld_1;
        LightingData param_33 = d;
        vec3 param_34 = outLighting0;
        vec3 param_35 = outLighting1;
        vec4 param_36 = vec4(1.0, 0.0, 0.0, 0.0);
        vec3 param_37 = vec3(1.0);
        calcLocalLightLighting(param_32, param_33, param_34, param_35, param_36, param_37);
        outLighting0 = param_34;
        outLighting1 = param_35;
    }
    outLighting0 += d.emissive;
    uint clusterIdx_2 = clusterIdxNonUniform;
    uint debugMode = floatBitsToUint(d.lightingWeights1.w);
    if (debugMode > 0u)
    {
        uint resourceOffset = debugMode - 1u;
        uint resourceCount = (_3468._resourceIndexBuffer[(clusterIdx_2 + (resourceOffset / 2u)) + (floatBitsToUint(_2399.frameParameters0.w) * 599040u)] >> uint(((resourceOffset % 2u) == 0u) ? 0 : 16)) & 65535u;
        if (resourceCount > 0u)
        {
            float param_38 = clamp(float(resourceCount) / 6.0, 0.0, 1.0);
            vec3 color = colormap(param_38).xyz;
            float maxResourceCountImportance = clamp(float(max((int(resourceCount) - 6), 0)) / 6.0, 0.0, 1.0);
            outLighting0 = mix(outLighting0, mix(color, vec3(1.0, 0.0, 1.0), vec3(maxResourceCountImportance)), vec3(0.5));
        }
    }
    vec3 _3870;
    if (false == all(equal(outLighting0, outLighting0)))
    {
        _3870 = vec3(1.0, 0.0, 0.0);
    }
    else
    {
        _3870 = outLighting0;
    }
    outLighting0 = _3870;
    vec3 _3882;
    if (false == all(equal(outLighting1, outLighting1)))
    {
        _3882 = vec3(1.0, 0.0, 0.0);
    }
    else
    {
        _3882 = outLighting1;
    }
    outLighting1 = _3882;
}

void main()
{
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.normal = inNormal;
    flrInput.tangent = inTangent;
    flrInput.binormal = inBinormal;
    flrInput.color = vec4(1.0);
    flrInput.posWS = inPosWS;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    LightingDataDesc desc;
    desc.baseColor = vec3(1.0);
    desc.roughness = flrOutput.parameter.x;
    desc.metalness = flrOutput.parameter.y;
    desc.occlusion = flrOutput.parameter.z;
    desc.translucency = flrOutput.parameter.w;
    desc.lightingWeights0 = _2399.lightingWeights0;
    desc.lightingWeights1 = _2399.lightingWeights1;
    desc.cameraPosWS = _2399.cameraPositionWS.xyz;
    desc.posWS = inPosWS;
    desc.posVS = (_2399.viewMatrix * vec4(inPosWS, 1.0)).xyz;
    vec2 param_2 = flrOutput.normals.zw;
    desc.NVtx = decodeNormal(param_2);
    vec2 param_3 = flrOutput.normals.xy;
    desc.N = decodeNormal(param_3);
    desc.ambientOcclusion = 1.0;
    desc.emissive = vec3(0.0);
    desc.irradiance = vec3(0.0);
    desc.materialId = _3970.params0.w;
    uint _4712 = desc.materialId;
    uint _4713 = clamp(_4712, 0u, 4096u);
    desc.materialData.paramIntData0 = _4279._materialDataBuffer[_4713].paramIntData0;
    desc.materialData.paramFloatData0 = _4279._materialDataBuffer[_4713].paramFloatData0;
    desc.materialData.paramFloatData1 = _4279._materialDataBuffer[_4713].paramFloatData1;
    desc.materialData.paramFloatData2 = _4279._materialDataBuffer[_4713].paramFloatData2;
    desc.materialData.paramFloatData3 = _4279._materialDataBuffer[_4713].paramFloatData3;
    desc.materialData.paramFloatData4 = _4279._materialDataBuffer[_4713].paramFloatData4;
    desc.materialData.paramFloatData5 = _4279._materialDataBuffer[_4713].paramFloatData5;
    desc.materialData.paramFloatData6 = _4279._materialDataBuffer[_4713].paramFloatData6;
    desc.materialData.paramFloatData7 = _4279._materialDataBuffer[_4713].paramFloatData7;
    desc.materialData.paramFloatData8 = _4279._materialDataBuffer[_4713].paramFloatData8;
    desc.materialData.paramFloatData9 = _4279._materialDataBuffer[_4713].paramFloatData9;
    desc.materialData.paramImgIDs0 = _4279._materialDataBuffer[_4713].paramImgIDs0;
    desc.materialData.paramImgIDs1 = _4279._materialDataBuffer[_4713].paramImgIDs1;
    desc.materialData.paramImgIDs2 = _4279._materialDataBuffer[_4713].paramImgIDs2;
    desc.materialBlendMask = vec4(1.0);
    LightingData d;
    LightingData param_4 = d;
    LightingDataDesc param_5 = desc;
    initLightingData(param_4, param_5);
    d = param_4;
    desc = param_5;
    d.T = inTangent.xyz;
    vec3 screenUV = inPosition.xyz / vec3(inPosition.w);
    vec2 _4770 = (screenUV.xy * 0.5) + vec2(0.5);
    screenUV = vec3(_4770.x, _4770.y, screenUV.z);
    d.irradiance = TEXTURE_LOD(irradianceTex, screenUV.xy * _2399.dynamicResolutionParameters.xy, 0.0).xyz;
    LightingData param_6 = d;
    vec3 param_7;
    vec3 param_8;
    calcLightingAll(param_6, param_7, param_8);
    d = param_6;
    vec3 outLighting0 = param_7;
    vec3 outLighting1 = param_8;
    outColor = vec4(outLighting0.x, outLighting0.y, outLighting0.z, outColor.w);
    outColor.w = 1.0;
}

