//HUD VS 4.
// Vk3DVision (http://3dsurroundgaming.com/) - Original Shader
#version 450
 #extension GL_EXT_multiview : enable
#define VIEW gl_ViewIndex
#extension GL_EXT_nonuniform_qualifier : require

struct FlrInput
{
    vec4 position;
    vec4 color;
    vec2 UV0;
    vec2 UV1; 
};

struct FlrOutput
{
    vec4 position;
    vec4 UV0;
    vec2 UV1;
    vec4 color;
    vec4 sdfParameters;
    vec4 colorOutline;
};

// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection

struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};


   

layout(set = 0, binding = 0, std140) uniform PerInstance
{
    mat4 projMatrix;
    mat4 viewProjMatrix;
    mat4 worldViewProjMatrix;
    vec4 sdfParsContour;
    vec4 sdfParsContourOutline;
    vec4 colorText;
    vec4 colorOutline;
    vec4 posOffset;
} _35;

layout(set = 0, binding = 99, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _292;

layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 0) in vec4 inPosition;
layout(location = 1) in vec4 inColor;
layout(location = 2) in vec2 inUV0;
layout(location = 3) in vec2 inUV1;
layout(location = 0) out vec4 outUV0;
layout(location = 1) out vec2 outUV1;
layout(location = 2) out vec4 outColor;
layout(location = 3) out vec4 outSdfParameters;
layout(location = 4) out vec4 outColorOutline;

FlrOutput flrMain(FlrInput flrInput)
{
    vec4 inputPos = flrInput.position;
    float sizeRelativeToNative = clamp(inputPos.w, 0.100000001490116119384765625, 10.0);
    float thickness = _35.sdfParsContour.x;
    float smoothing = _35.sdfParsContour.y;
    float thicknessOutline = _35.sdfParsContourOutline.x;
    float smoothingOutline = _35.sdfParsContourOutline.y;
    if (sizeRelativeToNative < 1.0)
    {
        float additionalSmoothing = (0.5 * smoothing) * (1.0 - sizeRelativeToNative);
        smoothing += additionalSmoothing;
        float additionalThickness = (0.20000000298023223876953125 * thickness) * (1.0 - sizeRelativeToNative);
        thickness += additionalThickness;
        float additionalSmoothingOutline = (0.5 * smoothingOutline) * (1.0 - sizeRelativeToNative);
        smoothingOutline += additionalSmoothingOutline;
        float additionalThicknessOutline = (0.20000000298023223876953125 * thicknessOutline) * (1.0 - sizeRelativeToNative);
        thicknessOutline += additionalThicknessOutline;
    }
    else
    {
        if (sizeRelativeToNative > 1.0)
        {
            float additionalSmoothing_1 = (0.20000000298023223876953125 * smoothing) * (1.0 - sizeRelativeToNative);
            smoothing += clamp(additionalSmoothing_1, (-smoothing) * 0.800000011920928955078125, 1.0);
            float additionalThickness_1 = (0.00650000013411045074462890625 * thickness) * (1.0 - sizeRelativeToNative);
            thickness += additionalThickness_1;
            float additionalSmoothingOutline_1 = (0.20000000298023223876953125 * smoothingOutline) * (1.0 - sizeRelativeToNative);
            smoothingOutline += clamp(additionalSmoothingOutline_1, (-smoothingOutline) * 0.800000011920928955078125, 1.0);
            float additionalThicknessOutline_1 = (0.00999999977648258209228515625 * thicknessOutline) * (1.0 - sizeRelativeToNative);
            thicknessOutline += additionalThicknessOutline_1;
        }
    }
    smoothing += flrInput.UV1.x;
    thickness += flrInput.UV1.y;
    thickness = clamp(thickness, 0.0, 0.9900000095367431640625);
    float sdf1 = 1.0 - thickness;
    float sdf2 = clamp(sdf1 + smoothing, 0.0, 1.0);
    float sdf3 = 1.0 - thicknessOutline;
    float sdf4 = clamp(sdf3 + smoothingOutline, 0.0, 1.0);
    FlrOutput flrOutput;
    flrOutput.sdfParameters = vec4(sdf1, sdf2, sdf3, sdf4);
    flrOutput.colorOutline = _35.colorOutline;
    vec3 _196 = inputPos.xyz * _35.posOffset.w;
    inputPos = vec4(_196.x, _196.y, _196.z, inputPos.w);
    inputPos.w = 1.0;
    flrOutput.position = _35.viewProjMatrix * inputPos;
    
   if (flrOutput.position.w==1 || flrOutput.position.w>50) 
    {
       flrOutput.position.x += vk3d_params[VIEW].stereo.x * (flrOutput.position.w - vk3d_params[VIEW].stereo.y);
    }
     else 
    {
	    flrOutput.position.x += vk3d_params[VIEW].stereo.x * flrOutput.position.w * vk3d_params[VIEW].custom_params.z;
    }
    
    flrOutput.color = flrInput.color;
    flrOutput.UV0 = vec4(flrInput.UV0.x, flrInput.UV0.y, flrOutput.UV0.z, flrOutput.UV0.w);
    flrOutput.UV0.z = 1.0;
    flrOutput.UV0.w = flrInput.position.w;
    flrOutput.UV1 = flrInput.UV1;
    return flrOutput;
}

void main()
{
    FlrInput flrInput;
    flrInput.position = inPosition;
    flrInput.color = inColor;
    flrInput.UV0 = inUV0;
    flrInput.UV1 = inUV1;
    FlrInput param = flrInput;
    FlrOutput flrOutput = flrMain(param);
    outUV0 = flrOutput.UV0;
    outUV1 = flrOutput.UV1;
    outColor = flrOutput.color;
    outSdfParameters = flrOutput.sdfParameters;
    outColorOutline = flrOutput.colorOutline;
    gl_Position = flrOutput.position;
}




