// SSAO pixel

#version 450
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require

struct UpsampleInput
{
    vec2 uv0;
    vec2 dynamicResolutionFactor;
};

struct FlrInput
{
    vec2 uv0;
    vec2 dynamicResolutionFactor;
};

struct FlrOutput
{
    vec4 outColor;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection

struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 110, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};


   

layout(set = 0, binding = 1, std140) uniform PerView
{
    vec4 dynamicResolutionParameters;
} _182;

layout(set = 0, binding = 100, std430) buffer DebugBuffer
{
    uint debugBuffer[];
} _219;

layout(set = 0, binding = 4) uniform SAMPLER_2D inputTex;
layout(set = 0, binding = 2) uniform SAMPLER_2D halfDepthBufferTex;
layout(set = 0, binding = 3) uniform SAMPLER_2D depthBufferTex;
layout(set = 2, binding = 0) uniform texture2D globalTextures[];
layout(set = 2, binding = 0) uniform textureCube globalCubeTextures[];
layout(set = 2, binding = 0) uniform texture3D global3DTextures[];
layout(set = 1, binding = 0) uniform sampler linearClampSampler;
layout(set = 1, binding = 1) uniform sampler linearRepeatSampler;
layout(set = 1, binding = 2) uniform sampler nearestClampSampler;
layout(set = 1, binding = 3) uniform sampler nearestRepeatSampler;
layout(set = 1, binding = 4) uniform sampler shadowSampler;

layout(location = 0) in vec2 inUV0;
layout(location = 0) out vec4 outColor;

vec4 nearestDepthUpsample(UpsampleInput upsampleInput)
{
    vec2 uv = upsampleInput.uv0;
    vec4 sampleR = TEXTURE_GATHER(inputTex, uv, 0);
    vec4 sampleG = TEXTURE_GATHER(inputTex, uv, 1);
    vec4 sampleB = TEXTURE_GATHER(inputTex, uv, 2);
    vec4 sampleA = TEXTURE_GATHER(inputTex, uv, 3);
    vec4 sampleDownsampledDepth = TEXTURE_GATHER(halfDepthBufferTex, uv, 0);
    float fullResDepth = TEXTURE_LOD(depthBufferTex, uv * upsampleInput.dynamicResolutionFactor, 0.0).x;
    float minDepthDiff = 1.0;
    uint nearestDepthIndex = 0u;
    float currentDepthDiff = abs(sampleDownsampledDepth.x - fullResDepth);
    bool rejectSample = currentDepthDiff < 9.9999999747524270787835121154785e-07;
    if (currentDepthDiff < minDepthDiff)
    {
        minDepthDiff = currentDepthDiff;
        nearestDepthIndex = 0u;
    }
    currentDepthDiff = abs(sampleDownsampledDepth.y - fullResDepth);
    rejectSample = rejectSample && (currentDepthDiff < 9.9999999747524270787835121154785e-07);
    if (currentDepthDiff < minDepthDiff)
    {
        minDepthDiff = currentDepthDiff;
        nearestDepthIndex = 1u;
    }
    currentDepthDiff = abs(sampleDownsampledDepth.z - fullResDepth);
    rejectSample = rejectSample && (currentDepthDiff < 9.9999999747524270787835121154785e-07);
    if (currentDepthDiff < minDepthDiff)
    {
        minDepthDiff = currentDepthDiff;
        nearestDepthIndex = 2u;
    }
    currentDepthDiff = abs(sampleDownsampledDepth.w - fullResDepth);
    rejectSample = rejectSample && (currentDepthDiff < 9.9999999747524270787835121154785e-07);
    if (currentDepthDiff < minDepthDiff)
    {
        minDepthDiff = currentDepthDiff;
        nearestDepthIndex = 3u;
    }
    if (rejectSample)
    {
        return TEXTURE_LOD(inputTex, uv, 0.0);
    }
    return vec4(sampleR[nearestDepthIndex], sampleG[nearestDepthIndex], sampleB[nearestDepthIndex], sampleA[nearestDepthIndex]);
}

void flrMain(FlrInput flrInput, inout FlrOutput flrOutput)
{
    UpsampleInput upsampleInput;
    upsampleInput.uv0 = flrInput.uv0;
    upsampleInput.dynamicResolutionFactor = flrInput.dynamicResolutionFactor;
    UpsampleInput param = upsampleInput;
    flrOutput.outColor = nearestDepthUpsample(param);
}

void main()
{
    FlrInput flrInput;
    flrInput.uv0 = inUV0;
    flrInput.dynamicResolutionFactor = _182.dynamicResolutionParameters.xy;
    FlrInput param = flrInput;
    FlrOutput param_1;
    flrMain(param, param_1);
    FlrOutput flrOutput = param_1;
    outColor = flrOutput.outColor;
}


